# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-10-23 12:29
# flake8: noqa: E303, F841
from __future__ import unicode_literals

from django.db import migrations
from django.core.management.sql import emit_post_migrate_signal



def update_permissions_for_group(apps, schema_editor):
    '''
    Update permissions for some users.

    Give bulk-delete permissions to moderators.
    Give edit permission to moderators and editors in order
    to display 'Main' page in the explorer.
    '''
    db_alias = schema_editor.connection.alias
    try:
        # Django 1.9
        emit_post_migrate_signal(2, False, db_alias)
    except TypeError:
        # Django < 1.9
        try:
            # Django 1.8
            emit_post_migrate_signal(2, False, 'default', db_alias)
        except TypeError:  # Django < 1.8
            emit_post_migrate_signal([], 2, False, 'default', db_alias)

    Group = apps.get_model('auth.Group')
    Permission = apps.get_model('auth.Permission')
    GroupPagePermission = apps.get_model('wagtailcore.GroupPagePermission')
    SectionIndexPage = apps.get_model('core.SectionIndexPage')
    MainPage = apps.get_model('core.Main')

    moderator_group = Group.objects.filter(name='Moderators').first()
    editor_group = Group.objects.filter(name='Editors').first()

    if moderator_group:
        sections = SectionIndexPage.objects.first()
        GroupPagePermission.objects.get_or_create(
            group_id=moderator_group.id,
            page_id=sections.id,
            permission_type='bulk_delete'
        )

        main = MainPage.objects.first()
        GroupPagePermission.objects.get_or_create(
            group_id=moderator_group.id,
            page_id=main.id,
            permission_type='edit'
        )

    if editor_group:
        main = MainPage.objects.first()
        GroupPagePermission.objects.get_or_create(
            group_id=editor_group.id,
            page_id=main.id,
            permission_type='edit'
        )


class Migration(migrations.Migration):
    dependencies = [
        ('core', '0051_remove_user_permission_for_moderator_group'),
        ('contenttypes', '0002_remove_content_type_name'),
        ('wagtailcore', '0032_add_bulk_delete_page_permission'),
        ('wagtailadmin', '0001_create_admin_access_permissions'),
        ('wagtailusers', '0005_make_related_name_wagtail_specific'),
        ('sites', '0002_alter_domain_unique'),
        ('auth', '0007_alter_validators_add_error_messages'),
    ]

    operations = [
        migrations.RunPython(
            update_permissions_for_group),
    ]



from wagtail.core.models import Page
from wagtail.core.utils import WAGTAIL_APPEND_SLASH
from django.urls import reverse


class MoloPage(Page):
    class Meta:
        proxy = True

    def get_url_parts(self, request=None):
        for (site_id, root_path, root_url) in self._get_site_root_paths(
                request):
            if hasattr(request, 'site') and site_id != request.site.pk:
                continue
            if self.url_path.startswith(root_path):
                page_path = reverse('wagtail_serve', args=(
                    self.url_path[len(root_path):],))

                # Remove the trailing slash from the URL reverse generates if
                # WAGTAIL_APPEND_SLASH is False and we're not trying to serve
                # the root path
                if not WAGTAIL_APPEND_SLASH and page_path != '/':
                    page_path = page_path.rstrip('/')
                return (site_id, root_url, page_path)



from {{cookiecutter.app_name}}.settings import *  # noqa

DATABASES = {
    'default': {
        'ENGINE': 'django_prometheus.db.backends.sqlite3',
        'NAME': '{{cookiecutter.app_name}}_test.db',
    }
}

DEBUG = True
CELERY_ALWAYS_EAGER = True

DEFAULT_SITE_PORT = 8000



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0023_alter_page_revision_on_delete_behaviour'),
        ('core', '0023_languagepage_main_language'),
    ]

    operations = [
        migrations.AddField(
            model_name='articlepage',
            name='language',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AddField(
            model_name='homepage',
            name='language',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AddField(
            model_name='sectionpage',
            name='language',
            field=models.CharField(max_length=255, blank=True),
        ),
    ]



# coding=utf-8
import pytest
from django.test import TestCase, RequestFactory
from mock import patch
from molo.core.models import (
    Main, SiteLanguageRelation, Languages, BannerPage, ArticlePageTags,
    FormPage)
from molo.core.tests.base import MoloTestCaseMixin
from molo.core.templatetags.core_tags import (
    get_parent, bannerpages, load_tags_for_article, get_recommended_articles,
    hero_article, render_translations
)
from molo.core.templatetags.forms_tags import forms_list


@pytest.mark.django_db
class TestModels(TestCase, MoloTestCaseMixin):

    def setUp(self):
        self.mk_main()
        self.main = Main.objects.all().first()
        self.factory = RequestFactory()
        self.language_setting = Languages.objects.create(
            site_id=self.main.get_site().pk)
        self.english = SiteLanguageRelation.objects.create(
            language_setting=self.language_setting,
            locale='en',
            is_active=True)

        self.french = SiteLanguageRelation.objects.create(
            language_setting=self.language_setting,
            locale='fr',
            is_active=True)

        self.yourmind = self.mk_section(
            self.section_index, title='Your mind')
        self.yourmind_sub = self.mk_section(
            self.yourmind, title='Your mind subsection')
        # create a requset object
        self.factory = RequestFactory()
        self.request = self.factory.get('/')
        self.request.site = self.site

    def create_form_page(
            self, parent, title="Test Form",
            slug="test-form", **kwargs):
        form_page = FormPage(
            title=title,
            slug=slug,
            intro='Introduction to Test Form ...',
            thank_you_text='Thank you for taking the Test Form',
            **kwargs
        )
        parent.add_child(instance=form_page)
        form_page.save_revision().publish()
        return form_page

    def test_get_form_list_homepage(self):
        context = {
            'locale_code': 'en',
            'request': self.request,
            'forms': self.create_form_page(self.form_index)
        }
        context = forms_list(context)
        self.assertEqual(len(context['forms']), 1)

    def test_render_translations(self):
        # this should return an empty dictionary for non main lang pages
        article = self.mk_articles(self.yourmind, 1)[0]
        fr_article = self.mk_article_translation(article, self.french)
        self.assertEquals(render_translations({}, fr_article), {})

    def test_bannerpages_without_position(self):
        banner = BannerPage(title='test banner')
        self.banner_index.add_child(instance=banner)
        banner.save_revision().publish()
        banner2 = BannerPage(title='test banner 2')
        self.banner_index.add_child(instance=banner2)
        banner2.save_revision().publish()
        banner3 = BannerPage(title='test banner 3')
        self.banner_index.add_child(instance=banner3)
        banner3.save_revision().publish()
        self.assertEqual(self.main.bannerpages().count(), 3)

        request = self.factory.get('/')
        request.site = self.site

        self.assertEquals(len(bannerpages({
            'locale_code': 'en', 'request': request})['bannerpages']), 3)

    def test_bannerpages_with_position(self):
        banner = BannerPage(title='test banner')
        self.banner_index.add_child(instance=banner)
        banner.save_revision().publish()
        banner2 = BannerPage(title='test banner 2')
        self.banner_index.add_child(instance=banner2)
        banner2.save_revision().publish()
        banner3 = BannerPage(title='test banner 3')
        self.banner_index.add_child(instance=banner3)
        banner3.save_revision().publish()
        self.assertEqual(self.main.bannerpages().count(), 3)

        request = self.factory.get('/')
        request.site = self.site

        self.assertEquals(len(bannerpages({
            'locale_code': 'en',
            'request': request}, position=0)['bannerpages']), 1)
        self.assertEquals(bannerpages({
            'locale_code': 'en',
            'request': request}, position=0)['bannerpages'][0].title,
            'test banner')
        self.assertEquals(bannerpages({
            'locale_code': 'en',
            'request': request}, position=1)['bannerpages'][0].title,
            'test banner 2')

    def test_bannerpages_with_position_out_of_range(self):
        banner = BannerPage(title='test banner')
        self.banner_index.add_child(instance=banner)
        banner.save_revision().publish()
        banner2 = BannerPage(title='test banner 2')
        self.banner_index.add_child(instance=banner2)
        banner2.save_revision().publish()
        banner3 = BannerPage(title='test banner 3')
        self.banner_index.add_child(instance=banner3)
        banner3.save_revision().publish()
        self.assertEqual(self.main.bannerpages().count(), 3)

        request = self.factory.get('/')
        request.site = self.site

        self.assertEquals(bannerpages({
            'locale_code': 'en',
            'request': request}, position=4), None)

    def test_get_parent_template_tag(self):
        request = self.factory.get('/')
        request.site = self.site

        article = self.mk_articles(self.yourmind, 1)[0]
        fr_article = self.mk_article_translation(article, self.french)

        self.assertEquals(
            get_parent({'locale_code': 'fr', 'request': request}, article),
            self.yourmind)
        self.assertEquals(
            get_parent({'locale_code': 'fr', 'request': request}, fr_article),
            self.yourmind)
        self.assertEquals(get_parent(
            {'locale_code': 'fr', 'request': request}, self.yourmind_sub),
            self.yourmind)

        fr_yourmind = self.mk_section_translation(self.yourmind, self.french)
        self.assertEquals(
            get_parent({'locale_code': 'en', 'request': request}, article),
            self.yourmind)
        self.assertEquals(
            get_parent({'locale_code': 'en', 'request': request}, fr_article),
            self.yourmind)
        self.assertEquals(get_parent(
            {'locale_code': 'en', 'request': request}, self.yourmind_sub),
            self.yourmind)

        self.assertEquals(
            get_parent({'locale_code': 'fr', 'request': request}, article),
            fr_yourmind)
        self.assertEquals(
            get_parent({'locale_code': 'fr', 'request': request}, fr_article),
            fr_yourmind)
        self.assertEquals(get_parent(
            {'locale_code': 'fr', 'request': request}, self.yourmind_sub),
            fr_yourmind)

        self.assertEquals(get_parent(
            {'locale_code': 'fr', 'request': request}, self.yourmind),
            None)

    def test_load_tags_for_article(self):
        request = self.factory.get('/')
        request.site = self.site
        article1 = self.mk_article(self.yourmind, title='article 1')

        tag = self.mk_tag(parent=self.tag_index)
        ArticlePageTags.objects.create(page=article1, tag=tag)
        self.assertEquals(load_tags_for_article(
            {
                'locale_code': 'en',
                'request': request
            }, article1)[0],
            tag)
        self.assertEquals(load_tags_for_article(
            {
                'locale_code': 'en',
                'request': request
            }, self.yourmind),
            None)

    def test_get_recommended_articles(self):
        request = self.factory.get('/')
        request.site = self.site
        article1 = self.mk_article(self.yourmind, title='article 1')

        self.assertEquals(get_recommended_articles(
            {'locale_code': 'en', 'request': request}, article1),
            [])

    @patch('molo.core.templatetags.core_tags.get_pages')
    def test_hero_article_empty_queryset_if_no_site(self, get_pages_mock):
        request = self.factory.get('/')
        request.site = None
        context = {'request': request, 'locale_code': 'en'}
        get_pages_mock.return_value = []

        self.assertEquals(
            hero_article(context),
            {
                'articles': [],
                'request': request,
                'locale_code': 'en',
            }
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import csv
from django.core.management.base import BaseCommand
from molo.core.models import Languages, ArticlePage, Main


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('csv_name', type=str)

    def handle(self, **options):
        csv_name = options.get('csv_name', None)
        mains = Main.objects.all()
        dates = {}
        with open(csv_name) as dates_csv:
            reader = csv.DictReader(dates_csv)
            if mains:
                for row in reader:
                    dates[row['slug']] = row['date']
        for main in mains:
            main_lang = Languages.for_site(main.get_site()).languages.filter(
                is_active=True, is_main_language=True).first()
            if main_lang:
                add_dates(self, main_lang, main, dates)
            else:
                self.stdout.write(self.style.NOTICE(
                    'Main language does not exist in "%s"' % main))


def add_dates(self, main_lang, main, dates):
    for slug in dates.keys():
        article = ArticlePage.objects.descendant_of(main).filter(slug=slug)
        if article.exists():
            article.update(featured_in_latest_start_date=dates[slug])



# -*- coding: utf-8 -*-
# flake8: noqa: E128
# Generated by Django 1.9.9 on 2016-10-23 12:29
from __future__ import unicode_literals

from django.db import migrations
from django.core.management.sql import emit_post_migrate_signal


def add_core_permissions_to_groups(apps, schema_editor):
    db_alias = schema_editor.connection.alias
    try:
        # Django 1.9
        emit_post_migrate_signal(2, False, db_alias)
    except TypeError:
        # Django < 1.9
        try:
            # Django 1.8
            emit_post_migrate_signal(2, False, 'default', db_alias)
        except TypeError:  # Django < 1.8
            emit_post_migrate_signal([], 2, False, 'default', db_alias)

    Group = apps.get_model('auth.Group')
    Permission = apps.get_model('auth.Permission')
    GroupPagePermission = apps.get_model('wagtailcore.GroupPagePermission')
    BannerIndexPage = apps.get_model('core.BannerIndexPage')
    SectionIndexPage = apps.get_model('core.SectionIndexPage')
    FooterIndexPage = apps.get_model('core.FooterIndexPage')

    if Group.objects.all().filter(name='Writer and Content Loader'):
        Group.objects.get(name='Writer and Content Loader').delete()

    if Group.objects.all().filter(name='Publisher'):
        Group.objects.get(name='Publisher').delete()

    # Create groups
    access_admin = Permission.objects.get(codename='access_admin')

    # <- Wagtail Login Only ->
    wagtail_login_only_group, _created = Group.objects.get_or_create(
        name='Wagtail Login Only')
    wagtail_login_only_group.permissions.add(access_admin)

    # <- Editors ->
    editor_group = Group.objects.get(name='Editors')

    # Remove the existing page permissions
    editor_group.page_permissions.all().delete()

    # Add page permissions
    sections = SectionIndexPage.objects.first()
    GroupPagePermission.objects.get_or_create(
        group=editor_group,
        page=sections,
        permission_type='add',
    )
    GroupPagePermission.objects.get_or_create(
        group=editor_group,
        page=sections,
        permission_type='edit',
    )

    banners = BannerIndexPage.objects.first()
    GroupPagePermission.objects.get_or_create(
        group=editor_group,
        page=banners,
        permission_type='add',
    )
    GroupPagePermission.objects.get_or_create(
        group=editor_group,
        page=banners,
        permission_type='edit',
    )

    add_sitelanguage = Permission.objects.get(codename='add_sitelanguage')
    editor_group.permissions.add(add_sitelanguage)

    # <- Moderator ->

    moderator_group = Group.objects.get(name='Moderators')

    add_sitelanguage = Permission.objects.get(codename='add_sitelanguage')
    change_sitelanguage = Permission.objects.get(
        codename='change_sitelanguage')
    change_user = Permission.objects.get(
        codename='change_user')
    moderator_group.permissions.add(add_sitelanguage, change_sitelanguage,
        change_user)

    moderator_group.page_permissions.all().delete()
    sections = SectionIndexPage.objects.first()
    GroupPagePermission.objects.get_or_create(
        group=moderator_group,
        page=sections,
        permission_type='add',
    )
    GroupPagePermission.objects.get_or_create(
        group=moderator_group,
        page=sections,
        permission_type='edit',
    )
    GroupPagePermission.objects.get_or_create(
        group=moderator_group,
        page=sections,
        permission_type='publish',
    )

    banners = BannerIndexPage.objects.first()
    GroupPagePermission.objects.get_or_create(
        group=moderator_group,
        page=banners,
        permission_type='add',
    )
    GroupPagePermission.objects.get_or_create(
        group=moderator_group,
        page=banners,
        permission_type='edit',
    )
    GroupPagePermission.objects.get_or_create(
        group=moderator_group,
        page=banners,
        permission_type='publish',
    )

    footers = FooterIndexPage.objects.first()
    GroupPagePermission.objects.get_or_create(
        group=moderator_group,
        page=footers,
        permission_type='add',
    )
    GroupPagePermission.objects.get_or_create(
        group=moderator_group,
        page=footers,
        permission_type='edit',
    )
    GroupPagePermission.objects.get_or_create(
        group=moderator_group,
        page=footers,
        permission_type='publish',
    )


class Migration(migrations.Migration):
    dependencies = [
        ('core', '0046_sitesettings_enable_clickable_tags'),
        ('contenttypes', '0002_remove_content_type_name'),
        ('wagtailcore', '0032_add_bulk_delete_page_permission'),
        ('wagtailadmin', '0001_create_admin_access_permissions'),
        ('wagtailusers', '0005_make_related_name_wagtail_specific'),
        ('sites', '0002_alter_domain_unique'),
        ('auth', '0007_alter_validators_add_error_messages'),
    ]

    operations = [
        migrations.RunPython(add_core_permissions_to_groups),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-03-28 07:49
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('profiles', '0012_userprofilessettings_terms_and_conditions'),
    ]

    operations = [
        migrations.AddField(
            model_name='userprofile',
            name='education_level',
            field=models.CharField(blank=True, max_length=128, null=True),
        ),
        migrations.AddField(
            model_name='userprofile',
            name='gender',
            field=models.CharField(blank=True, max_length=128, null=True),
        ),
        migrations.AddField(
            model_name='userprofile',
            name='location',
            field=models.CharField(blank=True, max_length=128, null=True),
        ),
        migrations.AddField(
            model_name='userprofilessettings',
            name='activate_display_name',
            field=models.BooleanField(default=True, verbose_name='Activate Display Name'),
        ),
        migrations.AddField(
            model_name='userprofilessettings',
            name='activate_dob',
            field=models.BooleanField(default=True, verbose_name='Activate Date Of Birth'),
        ),
        migrations.AddField(
            model_name='userprofilessettings',
            name='activate_education_level',
            field=models.BooleanField(default=False, verbose_name='Activate Education Level'),
        ),
        migrations.AddField(
            model_name='userprofilessettings',
            name='activate_education_level_required',
            field=models.BooleanField(default=False, verbose_name='Education Level Required'),
        ),
        migrations.AddField(
            model_name='userprofilessettings',
            name='activate_gender',
            field=models.BooleanField(default=False, verbose_name='Activate Gender'),
        ),
        migrations.AddField(
            model_name='userprofilessettings',
            name='activate_location',
            field=models.BooleanField(default=False, verbose_name='Activate Location'),
        ),
        migrations.AddField(
            model_name='userprofilessettings',
            name='capture_display_name_on_reg',
            field=models.BooleanField(default=False, help_text='If Display Name is activated, and Capture On Registration is not activated, The display name field will be captured on done page.', verbose_name='Capture On Registration'),
        ),
        migrations.AddField(
            model_name='userprofilessettings',
            name='capture_dob_on_reg',
            field=models.BooleanField(default=False, help_text='If Date Of Birth is activated, and Capture On Registration is not activated, The Date Of Birth field will be captured on done page.', verbose_name='Capture On Registration'),
        ),
        migrations.AddField(
            model_name='userprofilessettings',
            name='capture_education_level_on_reg',
            field=models.BooleanField(default=False, help_text='If Education Level is activated, and Capture On Registration is not activated, The Education Level field will be captured on done page.', verbose_name='Capture On Registration'),
        ),
        migrations.AddField(
            model_name='userprofilessettings',
            name='capture_gender_on_reg',
            field=models.BooleanField(default=False, help_text='If Gender is activated, and Capture On Registration is not activated, The Gender field will be captured on done page.', verbose_name='Capture On Registration'),
        ),
        migrations.AddField(
            model_name='userprofilessettings',
            name='capture_location_on_reg',
            field=models.BooleanField(default=False, help_text='If Location is activated, and Capture On Registration is not activated, The Location field will be captured on done page.', verbose_name='Capture On Registration'),
        ),
        migrations.AddField(
            model_name='userprofilessettings',
            name='display_name_required',
            field=models.BooleanField(default=False, verbose_name='Display Name Required'),
        ),
        migrations.AddField(
            model_name='userprofilessettings',
            name='dob_required',
            field=models.BooleanField(default=False, verbose_name='Date Of Birth Required'),
        ),
        migrations.AddField(
            model_name='userprofilessettings',
            name='gender_required',
            field=models.BooleanField(default=False, verbose_name='Gender Required'),
        ),
        migrations.AddField(
            model_name='userprofilessettings',
            name='location_required',
            field=models.BooleanField(default=False, verbose_name='Location Required'),
        ),
    ]



# -*- coding: utf-8 -*-
"""
Django settings for base {{cookiecutter.app_name}}.

For more information on this file, see
https://docs.djangoproject.com/en/1.7/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.7/ref/settings/
"""

from os.path import abspath, dirname, join
from os import environ
from django.conf import global_settings, locale
from django.utils.translation import ugettext_lazy as _
import dj_database_url
import djcelery
from celery.schedules import crontab
djcelery.setup_loader()

# Absolute filesystem path to the Django project directory:
PROJECT_ROOT = dirname(dirname(dirname(abspath(__file__))))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.7/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "{{cookiecutter.secret_key}}"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True
ENV = 'dev'

# List of hostnames which Django checks when serving your app, to
# prevent against HTTP Host header attacks.
ALLOWED_HOSTS = []


# Base URL to use when referring to full URLs within the Wagtail admin
# backend - e.g. in notification emails. Don't include '/admin' or
# a trailing slash
BASE_URL = 'http://example.com'


# Application definition

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.admin',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sites',
    'django_extensions',
    'django_prometheus',

    'taggit',
    'modelcluster',

    '{{cookiecutter.app_name}}',
    'molo.core',
    'molo.profiles',
    'mote',
    'google_analytics',

    'wagtail.core',
    'wagtail.admin',
    'wagtail.documents',
    'wagtail.snippets',
    'wagtail.users',
    'wagtail.sites',
    'wagtail.images',
    'wagtail.embeds',
    'wagtail.search',
    'wagtail.contrib.redirects',
    'wagtail.contrib.forms',
    'wagtailmedia',
    'wagtail.contrib.sitemaps',
    'wagtail.contrib.settings',
    'wagtail.contrib.modeladmin',
    'wagtail.api.v2',

    'mptt',
    'djcelery',
    'el_pagination',
{% for app_name, _ in cookiecutter.include %}    '{{app_name}}',
{% endfor %}
    'raven.contrib.django.raven_compat',
    'django_cas_ng',
    'compressor',
]

SITE_ID = 1
DEFAULT_SITE_PORT = 8000

MIDDLEWARE_CLASSES = [
    'django_prometheus.middleware.PrometheusBeforeMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'molo.core.middleware.ForceDefaultLanguageMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',

    'wagtail.core.middleware.SiteMiddleware',
    'wagtail.contrib.redirects.middleware.RedirectMiddleware',

    'molo.core.middleware.AdminLocaleMiddleware',
    'molo.core.middleware.NoScriptGASessionMiddleware',

    'molo.core.middleware.MoloGoogleAnalyticsMiddleware',
    'molo.core.middleware.MultiSiteRedirectToHomepage',
    'django_prometheus.middleware.PrometheusAfterMiddleware',
]

AUTHENTICATION_BACKENDS = [
    'molo.profiles.backends.MoloProfilesModelBackend',
    'molo.core.backends.MoloModelBackend',
    'django.contrib.auth.backends.ModelBackend'
]

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            join(PROJECT_ROOT, '{{cookiecutter.app_name}}', 'templates'),
        ],
        'APP_DIRS': False,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'molo.core.context_processors.locale',
                'molo.core.processors.compress_settings',
                'wagtail.contrib.settings.context_processors.settings',
            ],
            "loaders": [
                "django.template.loaders.filesystem.Loader",
                "mote.loaders.app_directories.Loader",
                "django.template.loaders.app_directories.Loader",

            ]
        },
    },
]

ROOT_URLCONF = '{{cookiecutter.app_name}}.urls'
WSGI_APPLICATION = '{{cookiecutter.app_name}}.wsgi.application'

# Google analytics

GOOGLE_ANALYTICS = {
    'google_analytics_id': None,
}

GOOGLE_ANALYTICS_IGNORE_PATH = [
    # health check used by marathon
    '/health/',
    # admin interfaces for wagtail and django
    '/admin/', '/django-admin/',
    # Universal Core content import URL
    '/import/',
    # browser troll paths
    '/favicon.ico', '/robots.txt',
    # when using nginx, we handle statics and media
    # but including them here just incase
    '/media/', '/static/',
    # metrics URL used by promethius monitoring system
    '/metrics/',
]

# Database
# https://docs.djangoproject.com/en/1.7/ref/settings/#databases

# SQLite (simplest install)
DATABASES = {
    'default': dj_database_url.config(
        default='sqlite:///%s' % (join(PROJECT_ROOT, 'db.sqlite3')),
        engine='django_prometheus.db.backends.sqlite3')
 }

DATABASES['default']['TEST'] = {}
DATABASES['default']['TEST']['NAME'] = join(PROJECT_ROOT, 'db.sqlite3')

# PostgreSQL (Recommended, but requires the psycopg2 library and Postgresql
#             development headers)
# DATABASES = {
#     'default': {
#         'ENGINE': 'django.db.backends.postgresql_psycopg2',
#         'NAME': 'base',
#         'USER': 'postgres',
#         'PASSWORD': '',
#         'HOST': '',  # Set to empty string for localhost.
#         'PORT': '',  # Set to empty string for default.
#         # number of seconds database connections should persist for
#         'CONN_MAX_AGE': 600,
#     }
# }

CELERY_ACCEPT_CONTENT = ['json']
CELERY_TASK_SERIALIZER = 'json'
CELERY_RESULT_SERIALIZER = 'json'
CELERY_IMPORTS = ('molo.core.tasks')
BROKER_URL = environ.get('BROKER_URL', 'redis://localhost:6379/0')
CELERY_RESULT_BACKEND = environ.get(
    'CELERY_RESULT_BACKEND', 'redis://localhost:6379/0')
CELERYBEAT_SCHEDULE = {
    'rotate_content': {
        'task': 'molo.core.tasks.rotate_content',
        'schedule': crontab(minute=0),
    },
    'molo_consolidated_minute_task': {
        'task': 'molo.core.tasks.molo_consolidated_minute_task',
        'schedule': crontab(minute='*'),
    },
}

# Internationalization
# https://docs.djangoproject.com/en/1.7/topics/i18n/
LANGUAGE_CODE = environ.get('LANGUAGE_CODE', 'en')
TIME_ZONE = 'UTC'
USE_I18N = True
USE_L10N = True
USE_TZ = True

# Native South African languages are currently not included in the default
# list of languges in django
# https://github.com/django/django/blob/master/django/conf/global_settings.py#L50
LANGUAGES = global_settings.LANGUAGES + [
    ('zu', _('Zulu')),
    ('xh', _('Xhosa')),
    ('st', _('Sotho')),
    ('ve', _('Venda')),
    ('tn', _('Tswana')),
    ('ts', _('Tsonga')),
    ('ss', _('Swati')),
    ('nr', _('Ndebele')),
]

EXTRA_LANG_INFO = {
    'zu': {
        'bidi': False,
        'code': 'zu',
        'name': 'Zulu',
        'name_local': 'isiZulu',
    },
    'xh': {
        'bidi': False,
        'code': 'xh',
        'name': 'Xhosa',
        'name_local': 'isiXhosa',
    },
    'st': {
        'bidi': False,
        'code': 'st',
        'name': 'Sotho',
        'name_local': 'seSotho',
    },
    've': {
        'bidi': False,
        'code': 've',
        'name': 'Venda',
        'name_local': u'tshiVenḓa',
    },
    'tn': {
        'bidi': False,
        'code': 'tn',
        'name': 'Tswana',
        'name_local': 'Setswana',
    },
    'ts': {
        'bidi': False,
        'code': 'ts',
        'name': 'Tsonga',
        'name_local': 'xiTsonga',
    },
    'ss': {
        'bidi': False,
        'code': 'ss',
        'name': 'Swati',
        'name_local': 'siSwati',
    },
    'nr': {
        'bidi': False,
        'code': 'nr',
        'name': 'Ndebele',
        'name_local': 'isiNdebele',
    }
}

locale.LANG_INFO.update(EXTRA_LANG_INFO)

LOCALE_PATHS = [
    join(PROJECT_ROOT, "locale"),
]

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.7/howto/static-files/

STATIC_ROOT = join(PROJECT_ROOT, 'static')
STATIC_URL = '/static/'
COMPRESS_ENABLED = True

STATICFILES_DIRS = [
    join(PROJECT_ROOT, "{{cookiecutter.app_name}}/{{ STATIC_URL }}css/{{ ENV }}"),
]

STATICFILES_FINDERS = [
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    'compressor.finders.CompressorFinder',
]

MEDIA_ROOT = join(PROJECT_ROOT, 'media')
MEDIA_URL = '/media/'
WAGTAILMEDIA_MEDIA_MODEL = 'core.MoloMedia'

# Django compressor settings
# http://django-compressor.readthedocs.org/en/latest/settings/

COMPRESS_PRECOMPILERS = [
    ('text/x-scss', 'django_libsass.SassCompiler'),
]


# Wagtail settings
LOGIN_URL = 'wagtailadmin_login'
LOGIN_REDIRECT_URL = 'wagtailadmin_home'

WAGTAIL_SITE_NAME = "base"

# Use Elasticsearch as the search backend for extra performance and better
# search results:
# http://wagtail.readthedocs.org/en/latest/howto/performance.html#search
# http://wagtail.readthedocs.org/en/latest/core_components/
#     search/backends.html#elasticsearch-backend
#
# WAGTAILSEARCH_BACKENDS = {
#     'default': {
#         'BACKEND': ('wagtail.search.backends.'
#                     'elasticsearch.ElasticSearch'),
#         'INDEX': 'base',
#     },
# }

# Explicitly declare our settings for logging
# This allows us to have custom logic for
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse',
        },
        'require_debug_true': {
            '()': 'django.utils.log.RequireDebugTrue',
        },
    },
    'handlers': {
        'console': {
            'level': 'INFO',
            'filters': ['require_debug_true'],
            'class': 'logging.StreamHandler',
        },
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        },
        'import_console': {
            'level': 'DEBUG',
            'filters': ['require_debug_true'],
            'class': 'logging.StreamHandler',
        },
    },
    'loggers': {
        'django': {
            'handlers': ['console', 'mail_admins'],
            'level': 'INFO',
        },
        'py.warnings': {
            'handlers': ['console'],
        },
        'import_logger': {
            'handlers': ['import_console'],
            'level': 'DEBUG',
            'propagate': True,
        },
    }
}


SITE_NAME = environ.get('SITE_NAME', "{{cookiecutter.app_name}}")
WAGTAIL_SITE_NAME = SITE_NAME

# Whether to use face/feature detection to improve image
# cropping - requires OpenCV
WAGTAILIMAGES_FEATURE_DETECTION_ENABLED = False

ENABLE_SSO = False

ADMIN_LANGUAGE_CODE = environ.get('ADMIN_LANGUAGE_CODE', "en")

FROM_EMAIL = environ.get('FROM_EMAIL', "support@moloproject.org")

# SMTP Settings
EMAIL_HOST = environ.get('EMAIL_HOST', 'localhost')
EMAIL_PORT = environ.get('EMAIL_PORT', 25)
EMAIL_HOST_USER = environ.get('EMAIL_HOST_USER', '')
EMAIL_HOST_PASSWORD = environ.get('EMAIL_HOST_PASSWORD', '')

CSRF_FAILURE_VIEW = 'molo.core.views.csrf_failure'

FREE_BASICS_URL_FOR_CSRF_MESSAGE = environ.get(
    'FREE_BASICS_URL_FOR_CSRF_MESSAGE', '')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-02-08 19:17
from __future__ import unicode_literals

from django.core.management.sql import emit_pre_migrate_signal, emit_post_migrate_signal
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import modelcluster.contrib.taggit
import modelcluster.fields
import molo.core.blocks
import molo.core.models
import taggit.managers
import wagtail.core.blocks
import wagtail.core.fields
import wagtail.core.models
import wagtail.images.blocks
import wagtail.search.index


# Functions from the following migrations need manual copying.
# Move them and any dependencies into this file, then update the
# RunPython operations to refer to the local versions:
# molo.core.migrations.0002_create_homepage
# NOTE: this function is labelled homepage, but creates BannerPage instead.
# This is due to necessary changes when squashing the migrations.
# Instead of creating the HomePage and then renaming it to BannerPage,
# we create the BannerPage immediately and remove migration to
# rename the model.
def create_homepage(apps, schema_editor):
    # Get models
    ContentType = apps.get_model('contenttypes.ContentType')
    Page = apps.get_model('wagtailcore.Page')
    Site = apps.get_model('wagtailcore.Site')
    BannerPage = apps.get_model('core.BannerPage')

    # Delete the default homepage
    Page.objects.get(id=2).delete()

    # Create content type for homepage model
    bannerpage_content_type, created = ContentType.objects.get_or_create(
        model='bannerpage', app_label='core')

    # Create a new bannerpage
    bannerpage = BannerPage.objects.create(
        title="Homepage",
        slug='home',
        content_type=bannerpage_content_type,
        path='00010001',
        depth=2,
        numchild=0,
        url_path='/home/',
    )

    # Create a site with the new bannerpage set as the root
    Site.objects.create(
        hostname='localhost', root_page=bannerpage, is_default_site=True)


# molo.core.migrations.0004_configure_root_page
def configure_root_page(apps, schema_editor):
    # Get models
    ContentType = apps.get_model('contenttypes.ContentType')
    Site = apps.get_model('wagtailcore.Site')
    Main = apps.get_model('core.Main')
    BannerPage = apps.get_model('core.BannerPage')

    # Delete the default homepage
    BannerPage.objects.all().delete()

    # Create content type for main model
    main_content_type, created = ContentType.objects.get_or_create(
        model='main', app_label='core')

    # Create a new homepage
    main = Main.objects.create(
        title="Main",
        slug='main',
        content_type=main_content_type,
        path='00010001',
        depth=2,
        numchild=0,
        url_path='/home/',
    )

    # Create a site with the new homepage set as the root
    Site.objects.all().delete()
    Site.objects.create(
        hostname='localhost', root_page=main, is_default_site=True)


# molo.core.migrations.0034_create_index_pages
def create_banner_index(apps, schema_editor):
    from molo.core.models import BannerIndexPage, Main
    main = Main.objects.first()

    if main:
        banner_index = BannerIndexPage(title='Banners', slug='banners')
        main.add_child(instance=banner_index)
        banner_index.save_revision().publish()


def create_section_index(apps, schema_editor):
    from molo.core.models import SectionIndexPage, Main
    main = Main.objects.all().first()

    if main:
        section_index = SectionIndexPage(title='Sections', slug='sections')
        main.add_child(instance=section_index)
        section_index.save_revision().publish()


def create_footer_index(apps, schema_editor):
    from molo.core.models import FooterIndexPage, Main
    main = Main.objects.all().first()

    if main:
        footer_index = FooterIndexPage(
            title='Footer pages', slug='footer-pages')
        main.add_child(instance=footer_index)
        footer_index.save_revision().publish()


# molo.core.migrations.0047_add_core_permissions_to_groups
def add_core_permissions_to_groups(apps, schema_editor):
    db_alias = schema_editor.connection.alias
    try:
        # Django 1.9
        emit_post_migrate_signal(2, False, db_alias)
    except TypeError:
        # Django < 1.9
        try:
            # Django 1.8
            emit_post_migrate_signal(2, False, 'default', db_alias)
        except TypeError:  # Django < 1.8
            emit_post_migrate_signal([], 2, False, 'default', db_alias)

    Group = apps.get_model('auth.Group')
    Permission = apps.get_model('auth.Permission')
    GroupPagePermission = apps.get_model('wagtailcore.GroupPagePermission')
    BannerIndexPage = apps.get_model('core.BannerIndexPage')
    SectionIndexPage = apps.get_model('core.SectionIndexPage')
    FooterIndexPage = apps.get_model('core.FooterIndexPage')

    if Group.objects.all().filter(name='Writer and Content Loader'):
        Group.objects.get(name='Writer and Content Loader').delete()

    if Group.objects.all().filter(name='Publisher'):
        Group.objects.get(name='Publisher').delete()

    # Create groups
    access_admin = Permission.objects.get(codename='access_admin')

    # <- Wagtail Login Only ->
    wagtail_login_only_group, _created = Group.objects.get_or_create(
        name='Wagtail Login Only')
    wagtail_login_only_group.permissions.add(access_admin)

    # <- Editors ->
    editor_group = Group.objects.get(name='Editors')

    # Remove the existing page permissions
    editor_group.page_permissions.all().delete()

    # Add page permissions
    sections = SectionIndexPage.objects.first()
    GroupPagePermission.objects.get_or_create(
        group=editor_group,
        page=sections,
        permission_type='add',
    )
    GroupPagePermission.objects.get_or_create(
        group=editor_group,
        page=sections,
        permission_type='edit',
    )

    banners = BannerIndexPage.objects.first()
    GroupPagePermission.objects.get_or_create(
        group=editor_group,
        page=banners,
        permission_type='add',
    )
    GroupPagePermission.objects.get_or_create(
        group=editor_group,
        page=banners,
        permission_type='edit',
    )

    add_sitelanguage = Permission.objects.get(codename='add_sitelanguage')
    editor_group.permissions.add(add_sitelanguage)

    # <- Moderator ->

    moderator_group = Group.objects.get(name='Moderators')

    add_sitelanguage = Permission.objects.get(codename='add_sitelanguage')
    change_sitelanguage = Permission.objects.get(
        codename='change_sitelanguage')
    change_user = Permission.objects.get(
        codename='change_user')
    moderator_group.permissions.add(add_sitelanguage, change_sitelanguage,
                                    change_user)

    moderator_group.page_permissions.all().delete()
    sections = SectionIndexPage.objects.first()
    GroupPagePermission.objects.get_or_create(
        group=moderator_group,
        page=sections,
        permission_type='add',
    )
    GroupPagePermission.objects.get_or_create(
        group=moderator_group,
        page=sections,
        permission_type='edit',
    )
    GroupPagePermission.objects.get_or_create(
        group=moderator_group,
        page=sections,
        permission_type='publish',
    )

    banners = BannerIndexPage.objects.first()
    GroupPagePermission.objects.get_or_create(
        group=moderator_group,
        page=banners,
        permission_type='add',
    )
    GroupPagePermission.objects.get_or_create(
        group=moderator_group,
        page=banners,
        permission_type='edit',
    )
    GroupPagePermission.objects.get_or_create(
        group=moderator_group,
        page=banners,
        permission_type='publish',
    )

    footers = FooterIndexPage.objects.first()
    GroupPagePermission.objects.get_or_create(
        group=moderator_group,
        page=footers,
        permission_type='add',
    )
    GroupPagePermission.objects.get_or_create(
        group=moderator_group,
        page=footers,
        permission_type='edit',
    )
    GroupPagePermission.objects.get_or_create(
        group=moderator_group,
        page=footers,
        permission_type='publish',
    )


# molo.core.migrations.0050_data_migration_promoted_articles
def set_promote_start_date(apps, schema_editor):
    ArticlePage = apps.get_model("core", "ArticlePage")

    latest_articles = ArticlePage.objects.filter(featured_in_latest=True)
    homepage_articles = ArticlePage.objects.filter(featured_in_homepage=True)
    section_articles = ArticlePage.objects.filter(featured_in_section=True)

    for article in latest_articles:
        article.featured_in_latest_start_date = article.latest_revision_created_at
        article.save()
    for article in homepage_articles:
        article.featured_in_homepage_start_date = article.latest_revision_created_at
        article.save()
    for article in section_articles:
        article.featured_in_section_start_date = article.latest_revision_created_at
        article.save()


# molo.core.migrations.0051_remove_user_permission_for_moderator_group
def remove_user_permission_for_moderator_group(apps, schema_editor):
    db_alias = schema_editor.connection.alias
    try:
        # Django 1.9
        emit_post_migrate_signal(2, False, db_alias)
    except TypeError:
        # Django < 1.9
        try:
            # Django 1.8
            emit_post_migrate_signal(2, False, 'default', db_alias)
        except TypeError:  # Django < 1.8
            emit_post_migrate_signal([], 2, False, 'default', db_alias)

    Group = apps.get_model('auth.Group')
    Permission = apps.get_model('auth.Permission')

    # <- Moderator ->
    moderator_group = Group.objects.filter(name='Moderators').first()
    if moderator_group:
        change_user = Permission.objects.get(codename='change_user')
        moderator_group.permissions.remove(change_user)


# molo.core.migrations.0052_update_permissions_for_groups
def update_permissions_for_group(apps, schema_editor):
    '''
    Update permissions for some users.

    Give bulk-delete permissions to moderators.
    Give edit permission to moderators and editors in order
    to display 'Main' page in the explorer.
    '''
    db_alias = schema_editor.connection.alias
    try:
        # Django 1.9
        emit_post_migrate_signal(2, False, db_alias)
    except TypeError:
        # Django < 1.9
        try:
            # Django 1.8
            emit_post_migrate_signal(2, False, 'default', db_alias)
        except TypeError:  # Django < 1.8
            emit_post_migrate_signal([], 2, False, 'default', db_alias)

    Group = apps.get_model('auth.Group')
    GroupPagePermission = apps.get_model('wagtailcore.GroupPagePermission')
    SectionIndexPage = apps.get_model('core.SectionIndexPage')
    MainPage = apps.get_model('core.Main')

    moderator_group = Group.objects.filter(name='Moderators').first()
    editor_group = Group.objects.filter(name='Editors').first()

    if moderator_group:
        sections = SectionIndexPage.objects.first()
        GroupPagePermission.objects.get_or_create(
            group_id=moderator_group.id,
            page_id=sections.id,
            permission_type='bulk_delete'
        )

        main = MainPage.objects.first()
        GroupPagePermission.objects.get_or_create(
            group_id=moderator_group.id,
            page_id=main.id,
            permission_type='edit'
        )

    if editor_group:
        main = MainPage.objects.first()
        GroupPagePermission.objects.get_or_create(
            group_id=editor_group.id,
            page_id=main.id,
            permission_type='edit'
        )


# molo.core.migrations.0055_migrate_current_site_languages_to_language_relation
def convert_languages_to_site_language_relation(apps, schema_editor):
    from molo.core.models import SiteLanguage, SiteLanguageRelation, Main, Languages
    main = Main.objects.all().first()
    if main:
        site = main.get_site()
        if site:
            language_setting, _ = Languages.objects.get_or_create(
                site_id=site.pk)
            for language in SiteLanguage.objects.all():
                site_language_rel = SiteLanguageRelation(
                    sitelanguage_ptr=language,
                    language_setting=language_setting,
                    locale=language.locale,
                    is_active=language.is_active,
                    is_main_language=language.is_main_language)
                site_language_rel.save()


# molo.core.migrations.0057_create_tag_index_page
def create_tag_index(apps, schema_editor):
    from molo.core.models import TagIndexPage, Main
    main = Main.objects.all().first()

    if main:
        tag_index = TagIndexPage(title='Tags', slug='tags')
        main.add_child(instance=tag_index)
        tag_index.save_revision().publish()


# molo.core.migrations.0060_reaction_questions_index_page
def create_reaction_question_index(apps, schema_editor):
    from molo.core.models import ReactionQuestionIndexPage, Main
    main = Main.objects.all().first()

    if main:
        reaction_question_index = ReactionQuestionIndexPage(
            title='Reaction Questions', slug='reaction-questions')
        main.add_child(instance=reaction_question_index)
        reaction_question_index.save_revision().publish()


# molo.core.migrations.0068_media_migration
def convert_media_to_molo_media(apps, schema_editor):
    db_alias = schema_editor.connection.alias
    try:
        # Django 1.9
        emit_post_migrate_signal(2, False, db_alias)
    except TypeError:
        # Django < 1.9
        try:
            # Django 1.8
            emit_post_migrate_signal(2, False, 'default', db_alias)
        except TypeError:  # Django < 1.8
            emit_post_migrate_signal([], 2, False, 'default', db_alias)
    from molo.core.models import MoloMedia, ArticlePage
    from wagtailmedia.models import Media

    for media in Media.objects.all():
        if media.type == 'video':
            new_media = MoloMedia.objects.create(
                title=media.title, file=media.file, duration=media.duration,
                type=media.type, width=media.width, height=media.height,
                thumbnail=media.thumbnail)
        else:
            new_media = MoloMedia.objects.create(
                title=media.title, file=media.file, duration=media.duration,
                type=media.type)
        media.file = None
        media.save()
        for article in ArticlePage.objects.all():
            for block in article.body:
                if block.block_type is 'media' and block.value is media.id:
                    block.value = new_media.id
                    article.save()


# molo.core.migrations.0071_remove_old_image_hashes
def delete_imageinfo(apps, schema_editor):
    ImageInfo = apps.get_model('core.ImageInfo')
    ImageInfo.objects.all().delete()


# molo.core.migrations.0073_run_wagtail_migration_before_core_34
def run_wagtail_migration_before_core_34(apps, schema_editor):
    """
    Migration 34 needs migration 0040 from wagtail core
    and this Migration will run wagtail migration before
    molo core migration 34
    """
    db_alias = schema_editor.connection.alias
    emit_pre_migrate_signal(verbosity=2, interactive=False, db=db_alias)


# molo.core.migrations.0075_create_timezone
def seed_timezones(apps, schema_editor):
    from molo.core.models import Timezone
    from pytz import common_timezones
    for timezone_title in common_timezones:
        timezone = Timezone.objects.create(title=timezone_title)
        timezone.save()


def delete_timezones(apps, schema_editor):
    from molo.core.models import Timezone
    Timezone.objects.all().delete()


# molo.core.migrations.0076_create_cmssettings
def set_initial_timezone(apps, schema_editor):
    from django.conf import settings as django_settings
    from molo.core.models import CmsSettings, Timezone
    from wagtail.core.models import Site

    current_timezone_name = django_settings.TIME_ZONE or 'UTC'
    for site in Site.objects.all():
        cms_settings = CmsSettings.for_site(site)
        cms_settings.timezone = Timezone.objects.filter(
            title=current_timezone_name,
        ).first()
        cms_settings.save()


def unset_timezone(apps, schema_editor):
    from molo.core.models import CmsSettings
    from wagtail.core.models import Site
    for site in Site.objects.all():
        site_settings = CmsSettings.for_site(site)
        site_settings.timezone = None
        site_settings.save()


class Migration(migrations.Migration):

    replaces = [
        ('core', '0001_initial'),
        ('core', '0002_create_homepage'),
        ('core', '0003_articlepage_languagepage_main_sectionpage'),
        ('core', '0004_configure_root_page'),
        ('core', '0005_add_nullable_to_articlepage_sectionpage'),
        ('core', '0006_add_help_text_to_language_code'),
        ('core', '0007_articlepage_body'),
        ('core', '0008_articlepage_add_more_blocks'),
        ('core', '0009_sectionpage_image'),
        ('core', '0010_add_banner_to_homepage'),
        ('core', '0011_footerpage'),
        ('core', '0012_add_featured_fields_to_articlepage'),
        ('core', '0013_articlepage_featured_in_homepage'),
        ('core', '0014_sectionpage_extra_style_hints'),
        ('core', '0015_auto_20150925_1037'),
        ('core', '0016_auto_20150925_1226'),
        ('core', '0017_auto_20150925_1232'),
        ('core', '0018_auto_20150925_1554'),
        ('core', '0019_add_tags_to_article'),
        ('core', '0020_add-social-media-fields-to-article-page'),
        ('core', '0021_add-meta-tags-field-to-article-page'),
        ('core', '0022_add_help_text_for_metadata_tags'),
        ('core', '0023_languagepage_main_language'),
        ('core', '0024_add_language_field_to_models'),
        ('core', '0025_add_translation_model'),
        ('core', '0026_add_site_language_model'),
        ('core', '0027_remove_commenting_fields_from_homepage'),
        ('core', '0028_rename_homepage_to_bannerpage'),
        ('core', '0029_add_UUID_filed'),
        ('core', '0030_change_related_name_for_translations'),
        ('core', '0031_sitesettings'),
        ('core', '0032_sitesettings_ga_tag_manager'),
        ('core', '0033_bannerindexpage_footerindexpage_sectionindexpage'),
        ('core', '0034_create_index_pages'),
        ('core', '0035_content_rotation_settings'),
        ('core', '0036_auto_add_related_section_to_article'),
        ('core', '0037_add_article_body_media_field'),
        ('core', '0038_sitesettings_global_ga_tag_manager'),
        ('core', '0039_add_content_rotation_to_SectionPage'),
        ('core', '0040_auto_20160722_1041'),
        ('core', '0041_GA_tracking_code'),
        ('core', '0042_content_rotation_dates_and_times'),
        ('core', '0043_sitesettings_show_only_translated_pages'),
        ('core', '0044_add_topic_of_the_day_fields'),
        ('core', '0045_bannerpage_external_link'),
        ('core', '0046_sitesettings_enable_clickable_tags'),
        ('core', '0047_add_core_permissions_to_groups'),
        ('core', '0048_promotion_dates'),
        ('core', '0049_add_social_media_links_to_site_settings'),
        ('core', '0050_data_migration_promoted_articles'),
        ('core', '0051_remove_user_permission_for_moderator_group'),
        ('core', '0052_update_permissions_for_groups'),
        ('core', '0053_add_next_and_recommended_functionality'),
        ('core', '0054_languages_sitelanguagerelation'),
        ('core', '0055_migrate_current_site_languages_to_language_relation'),
        ('core', '0056_tag_navigation'),
        ('core', '0057_create_tag_index_page'),
        ('core', '0058_section_tags'),
        ('core', '0059_reaction_questions'),
        ('core', '0060_reaction_questions_index_page'),
        ('core', '0061_add_success_fields_to_reaction_choice'),
        ('core', '0062_increase_char_length_reaction_success'),
        ('core', '0063_articlepagelanguageproxy'),
        ('core', '0064_auto_20170802_1410'),
        ('core', '0065_auto_20170807_1005'),
        ('core', '0066_add_custom_media_model'),
        ('core', '0067_promote_homepage_media'),
        ('core', '0068_media_migration'),
        ('core', '0069_add_image_info'),
        ('core', '0070_add_service_directory_api'),
        ('core', '0071_remove_old_image_hashes'),
        ('core', '0072_fb_tracking_id'),
        ('core', '0073_run_wagtail_migration_before_core_34'),
        ('core', '0074_banner_subtitle'),
        ('core', '0075_create_timezone'),
        ('core', '0076_create_cmssettings'),
        ('core', '0077_molo_page')
    ]

    initial = True

    dependencies = [
        ('wagtailcore', '0015_add_more_verbose_names'),
        ('wagtailmedia', '0003_copy_media_permissions_to_collections'),
        ('wagtailimages', '0005_make_filter_spec_unique'),
        ('wagtailcore', '0029_unicode_slugfield_dj19'),
        ('auth', '0007_alter_validators_add_error_messages'),
        ('sites', '0002_alter_domain_unique'),
        ('wagtailimages', '0012_copy_image_permissions_to_collections'),
        ('taggit', '0002_auto_20150616_2121'),
        ('wagtailcore', '0010_change_page_owner_to_null_on_delete'),
        ('contenttypes', '0002_remove_content_type_name'),
        ('wagtailcore', '0028_merge'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('wagtailimages', '0006_add_verbose_names'),
        ('wagtailcore', '0040_page_draft_title'),
        ('taggit', '0001_initial'),
        ('wagtailcore', '0032_add_bulk_delete_page_permission'),
        ('wagtailcore', '0023_alter_page_revision_on_delete_behaviour'),
        ('wagtailusers', '0005_make_related_name_wagtail_specific'),
        ('wagtailimages', '0010_change_on_delete_behaviour'),
        ('wagtailcore', '0024_alter_page_content_type_on_delete_behaviour'),
        ('wagtailadmin', '0001_create_admin_access_permissions'),
        ('wagtailimages', '0018_remove_rendition_filter'),
    ]

    operations = [
        migrations.CreateModel(
            name='BannerPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
        migrations.RunPython(create_homepage),
        migrations.CreateModel(
            name='ArticlePage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('subtitle', models.TextField()),
                ('image', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.Image')),
            ],
            options={
                'verbose_name': 'Article',
            },
            bases=('wagtailcore.page',),
        ),
        migrations.CreateModel(
            name='LanguagePage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('code', models.CharField(max_length=255)),
            ],
            options={
                'verbose_name': 'Language',
            },
            bases=('wagtailcore.page',),
        ),
        migrations.CreateModel(
            name='Main',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
        migrations.CreateModel(
            name='SectionPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('description', models.TextField()),
            ],
            options={
                'verbose_name': 'Section',
            },
            bases=('wagtailcore.page',),
        ),
        migrations.RunPython(configure_root_page),
        migrations.AlterField(
            model_name='articlepage',
            name='subtitle',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='sectionpage',
            name='description',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='languagepage',
            name='code',
            field=models.CharField(help_text='The language code as specified in iso639-2', max_length=255),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='body',
            field=wagtail.core.fields.StreamField(((b'heading', wagtail.core.blocks.CharBlock(classname=b'full title')), (b'paragraph', molo.core.blocks.MarkDownBlock()), (b'image', wagtail.images.blocks.ImageChooserBlock()), (b'list', wagtail.core.blocks.ListBlock(wagtail.core.blocks.CharBlock(label=b'Item'))), (b'numbered_list', wagtail.core.blocks.ListBlock(wagtail.core.blocks.CharBlock(label=b'Item'))), (b'page', wagtail.core.blocks.PageChooserBlock())), blank=True, null=True),
        ),
        migrations.AddField(
            model_name='sectionpage',
            name='image',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.Image'),
        ),
        migrations.AddField(
            model_name='bannerpage',
            name='banner',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.Image'),
        ),
        migrations.AddField(
            model_name='bannerpage',
            name='banner_link_page',
            field=models.ForeignKey(blank=True, help_text='Optional page to which the banner will link to', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.Page'),
        ),
        migrations.CreateModel(
            name='FooterPage',
            fields=[
                ('articlepage_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.ArticlePage')),
            ],
            options={
                'abstract': False,
            },
            bases=('core.articlepage',),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='featured_in_latest',
            field=models.BooleanField(default=False, help_text='Article to be featured in the Latest module'),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='featured_in_section',
            field=models.BooleanField(default=False, help_text='Article to be featured in the Section module'),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='featured_in_homepage',
            field=models.BooleanField(default=False, help_text='Article to be featured in the Homepage within the Section module'),
        ),
        migrations.AddField(
            model_name='sectionpage',
            name='extra_style_hints',
            field=models.TextField(blank=True, default=b'', help_text='Styling options that can be applied to this section and all its descendants', null=True),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='commenting_state',
            field=models.CharField(blank=True, choices=[(b'O', b'Open'), (b'C', b'Closed'), (b'D', b'Disabled'), (b'T', b'Timestamped')], max_length=1, null=True),
        ),
        migrations.AddField(
            model_name='languagepage',
            name='commenting_state',
            field=models.CharField(blank=True, choices=[(b'O', b'Open'), (b'C', b'Closed'), (b'D', b'Disabled'), (b'T', b'Timestamped')], max_length=1, null=True),
        ),
        migrations.AddField(
            model_name='sectionpage',
            name='commenting_state',
            field=models.CharField(blank=True, choices=[(b'O', b'Open'), (b'C', b'Closed'), (b'D', b'Disabled'), (b'T', b'Timestamped')], max_length=1, null=True),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='commenting_close_time',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='commenting_open_time',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='languagepage',
            name='commenting_close_time',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='languagepage',
            name='commenting_open_time',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='sectionpage',
            name='commenting_close_time',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='sectionpage',
            name='commenting_open_time',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.CreateModel(
            name='ArticlePageTag',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('content_object', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='tagged_items', to='core.ArticlePage')),
                ('tag', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='core_articlepagetag_items', to='taggit.Tag')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='articlepage',
            name='tags',
            field=modelcluster.contrib.taggit.ClusterTaggableManager(blank=True, help_text='A comma-separated list of tags.', through='core.ArticlePageTag', to='taggit.Tag', verbose_name='Tags'),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='social_media_description',
            field=models.TextField(blank=True, null=True, verbose_name=b'description'),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='social_media_image',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.Image', verbose_name=b'Image'),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='social_media_title',
            field=models.TextField(blank=True, null=True, verbose_name=b'title'),
        ),
        migrations.CreateModel(
            name='ArticlePageMetaDataTag',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('content_object', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='metadata_tagged_items', to='core.ArticlePage')),
                ('tag', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='core_articlepagemetadatatag_items', to='taggit.Tag')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='articlepage',
            name='metadata_tags',
            field=modelcluster.contrib.taggit.ClusterTaggableManager(blank=True, help_text='A comma-separated list of tags. This is not visible to the user.', through='core.ArticlePageMetaDataTag', to='taggit.Tag', verbose_name='Tags'),
        ),
        migrations.CreateModel(
            name='PageTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('page', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='wagtailcore.Page')),
                ('translated_page', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='source_page', to='wagtailcore.Page')),
            ],
        ),
        migrations.CreateModel(
            name='LanguageRelation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
        ),
        migrations.CreateModel(
            name='SiteLanguage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('locale', models.CharField(choices=[(b'af', b'Afrikaans'), (b'ar', b'Arabic'), (b'ast', b'Asturian'), (b'az', b'Azerbaijani'), (b'bg', b'Bulgarian'), (b'be', b'Belarusian'), (b'bn', b'Bengali'), (b'br', b'Breton'), (b'bs', b'Bosnian'), (b'ca', b'Catalan'), (b'cs', b'Czech'), (b'cy', b'Welsh'), (b'da', b'Danish'), (b'de', b'German'), (b'el', b'Greek'), (b'en', b'English'), (b'en-au', b'Australian English'), (b'en-gb', b'British English'), (b'eo', b'Esperanto'), (b'es', b'Spanish'), (b'es-ar', b'Argentinian Spanish'), (b'es-mx', b'Mexican Spanish'), (b'es-ni', b'Nicaraguan Spanish'), (b'es-ve', b'Venezuelan Spanish'), (b'et', b'Estonian'), (b'eu', b'Basque'), (b'fa', b'Persian'), (b'fi', b'Finnish'), (b'fr', b'French'), (b'fy', b'Frisian'), (b'ga', b'Irish'), (b'gl', b'Galician'), (b'he', b'Hebrew'), (b'hi', b'Hindi'), (b'hr', b'Croatian'), (b'hu', b'Hungarian'), (b'ia', b'Interlingua'), (b'id', b'Indonesian'), (b'io', b'Ido'), (b'is', b'Icelandic'), (b'it', b'Italian'), (b'ja', b'Japanese'), (b'ka', b'Georgian'), (b'kk', b'Kazakh'), (b'km', b'Khmer'), (b'kn', b'Kannada'), (b'ko', b'Korean'), (b'lb', b'Luxembourgish'), (b'lt', b'Lithuanian'), (b'lv', b'Latvian'), (b'mk', b'Macedonian'), (b'ml', b'Malayalam'), (b'mn', b'Mongolian'), (b'mr', b'Marathi'), (b'my', b'Burmese'), (b'nb', b'Norwegian Bokmal'), (b'ne', b'Nepali'), (b'nl', b'Dutch'), (b'nn', b'Norwegian Nynorsk'), (b'os', b'Ossetic'), (b'pa', b'Punjabi'), (b'pl', b'Polish'), (b'pt', b'Portuguese'), (b'pt-br', b'Brazilian Portuguese'), (b'ro', b'Romanian'), (b'ru', b'Russian'), (b'sk', b'Slovak'), (b'sl', b'Slovenian'), (b'sq', b'Albanian'), (b'sr', b'Serbian'), (b'sr-latn', b'Serbian Latin'), (b'sv', b'Swedish'), (b'sw', b'Swahili'), (b'ta', b'Tamil'), (b'te', b'Telugu'), (b'th', b'Thai'), (b'tr', b'Turkish'), (b'tt', b'Tatar'), (b'udm', b'Udmurt'), (b'uk', b'Ukrainian'), (b'ur', b'Urdu'), (b'vi', b'Vietnamese'), (b'zh-cn', b'Simplified Chinese'), (b'zh-hans', b'Simplified Chinese'), (b'zh-hant', b'Traditional Chinese'), (b'zh-tw', b'Traditional Chinese'), (b'zu', 'Zulu'), (b'xh', 'Xhosa'), (b'st', 'Sotho'), (b've', 'Venda'), (b'tn', 'Tswana'), (b'ts', 'Tsonga'), (b'ss', 'Swati'), (b'nr', 'Ndebele')], help_text='Site language', max_length=255, verbose_name='language name')),
                ('is_main_language', models.BooleanField(default=False, editable=False, verbose_name='main Language')),
                ('is_active', models.BooleanField(default=True, verbose_name='active Language')),
            ],
            options={
                'verbose_name': 'Language',
            },
        ),
        migrations.AddField(
            model_name='languagerelation',
            name='language',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='core.SiteLanguage'),
        ),
        migrations.AddField(
            model_name='languagerelation',
            name='page',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='languages', to='wagtailcore.Page'),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='uuid',
            field=models.CharField(blank=True, max_length=32, null=True),
        ),
        migrations.AddField(
            model_name='sectionpage',
            name='uuid',
            field=models.CharField(blank=True, max_length=32, null=True),
        ),
        migrations.CreateModel(
            name='SiteSettings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('logo', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.Image')),
                ('site', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.Site')),
                ('ga_tag_manager', models.CharField(blank=True, help_text='GA Tag Manager tracking code (e.g GTM-XXX)', max_length=255, null=True, verbose_name='GA Tag Manager')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='BannerIndexPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
        migrations.CreateModel(
            name='FooterIndexPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
        migrations.CreateModel(
            name='SectionIndexPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('commenting_state', models.CharField(blank=True, choices=[(b'O', b'Open'), (b'C', b'Closed'), (b'D', b'Disabled'), (b'T', b'Timestamped')], default=b'O', max_length=1, null=True)),
                ('commenting_open_time', models.DateTimeField(blank=True, null=True)),
                ('commenting_close_time', models.DateTimeField(blank=True, null=True)),
            ],
            options={
                'abstract': False,
            },
            bases=(molo.core.models.CommentedPageMixin, 'wagtailcore.page'),
        ),
        migrations.RunPython(create_banner_index),
        migrations.RunPython(create_section_index),
        migrations.RunPython(create_footer_index),
        migrations.CreateModel(
            name='ArticlePageRelatedSections',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('page', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='related_sections', to='core.ArticlePage')),
                ('section', models.ForeignKey(blank=True, help_text='Section that this page also belongs too', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.Page')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
        migrations.AlterField(
            model_name='sitelanguage',
            name='locale',
            field=models.CharField(choices=[(b'af', b'Afrikaans'), (b'ar', b'Arabic'), (b'ast', b'Asturian'), (b'az', b'Azerbaijani'), (b'bg', b'Bulgarian'), (b'be', b'Belarusian'), (b'bn', b'Bengali'), (b'br', b'Breton'), (b'bs', b'Bosnian'), (b'ca', b'Catalan'), (b'cs', b'Czech'), (b'cy', b'Welsh'), (b'da', b'Danish'), (b'de', b'German'), (b'el', b'Greek'), (b'en', b'English'), (b'en-au', b'Australian English'), (b'en-gb', b'British English'), (b'eo', b'Esperanto'), (b'es', b'Spanish'), (b'es-ar', b'Argentinian Spanish'), (b'es-co', b'Colombian Spanish'), (b'es-mx', b'Mexican Spanish'), (b'es-ni', b'Nicaraguan Spanish'), (b'es-ve', b'Venezuelan Spanish'), (b'et', b'Estonian'), (b'eu', b'Basque'), (b'fa', b'Persian'), (b'fi', b'Finnish'), (b'fr', b'French'), (b'fy', b'Frisian'), (b'ga', b'Irish'), (b'gd', b'Scottish Gaelic'), (b'gl', b'Galician'), (b'he', b'Hebrew'), (b'hi', b'Hindi'), (b'hr', b'Croatian'), (b'hu', b'Hungarian'), (b'ia', b'Interlingua'), (b'id', b'Indonesian'), (b'io', b'Ido'), (b'is', b'Icelandic'), (b'it', b'Italian'), (b'ja', b'Japanese'), (b'ka', b'Georgian'), (b'kk', b'Kazakh'), (b'km', b'Khmer'), (b'kn', b'Kannada'), (b'ko', b'Korean'), (b'lb', b'Luxembourgish'), (b'lt', b'Lithuanian'), (b'lv', b'Latvian'), (b'mk', b'Macedonian'), (b'ml', b'Malayalam'), (b'mn', b'Mongolian'), (b'mr', b'Marathi'), (b'my', b'Burmese'), (b'nb', b'Norwegian Bokmal'), (b'ne', b'Nepali'), (b'nl', b'Dutch'), (b'nn', b'Norwegian Nynorsk'), (b'os', b'Ossetic'), (b'pa', b'Punjabi'), (b'pl', b'Polish'), (b'pt', b'Portuguese'), (b'pt-br', b'Brazilian Portuguese'), (b'ro', b'Romanian'), (b'ru', b'Russian'), (b'sk', b'Slovak'), (b'sl', b'Slovenian'), (b'sq', b'Albanian'), (b'sr', b'Serbian'), (b'sr-latn', b'Serbian Latin'), (b'sv', b'Swedish'), (b'sw', b'Swahili'), (b'ta', b'Tamil'), (b'te', b'Telugu'), (b'th', b'Thai'), (b'tr', b'Turkish'), (b'tt', b'Tatar'), (b'udm', b'Udmurt'), (b'uk', b'Ukrainian'), (b'ur', b'Urdu'), (b'vi', b'Vietnamese'), (b'zh-hans', b'Simplified Chinese'), (b'zh-hant', b'Traditional Chinese'), (b'zu', 'Zulu'), (b'xh', 'Xhosa'), (b'st', 'Sotho'), (b've', 'Venda'), (b'tn', 'Tswana'), (b'ts', 'Tsonga'), (b'ss', 'Swati'), (b'nr', 'Ndebele')], help_text='Site language', max_length=255, verbose_name='language name'),
        ),
        migrations.AlterField(
            model_name='articlepage',
            name='body',
            field=wagtail.core.fields.StreamField(((b'heading', wagtail.core.blocks.CharBlock(classname=b'full title')), (b'paragraph', molo.core.blocks.MarkDownBlock()), (b'image', wagtail.images.blocks.ImageChooserBlock()), (b'list', wagtail.core.blocks.ListBlock(wagtail.core.blocks.CharBlock(label=b'Item'))), (b'numbered_list', wagtail.core.blocks.ListBlock(wagtail.core.blocks.CharBlock(label=b'Item'))), (b'page', wagtail.core.blocks.PageChooserBlock()), (b'media', molo.core.blocks.MultimediaBlock())), blank=True, null=True),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='global_ga_tag_manager',
            field=models.CharField(blank=True, help_text='Global GA Tag Manager tracking code (e.g GTM-XXX) to be used to view analytics on more than one site globally', max_length=255, null=True, verbose_name='Global GA Tag Manager'),
        ),
        migrations.AlterField(
            model_name='sitesettings',
            name='ga_tag_manager',
            field=models.CharField(blank=True, help_text='Local GA Tag Manager tracking code (e.g GTM-XXX) to be used to view analytics on this site only', max_length=255, null=True, verbose_name='Local GA Tag Manager'),
        ),
        migrations.AlterField(
            model_name='sitelanguage',
            name='locale',
            field=models.CharField(choices=[(b'af', b'Afrikaans'), (b'ar', b'Arabic'), (b'ast', b'Asturian'), (b'az', b'Azerbaijani'), (b'bg', b'Bulgarian'), (b'be', b'Belarusian'), (b'bn', b'Bengali'), (b'br', b'Breton'), (b'bs', b'Bosnian'), (b'ca', b'Catalan'), (b'cs', b'Czech'), (b'cy', b'Welsh'), (b'da', b'Danish'), (b'de', b'German'), (b'el', b'Greek'), (b'en', b'English'), (b'en-au', b'Australian English'), (b'en-gb', b'British English'), (b'eo', b'Esperanto'), (b'es', b'Spanish'), (b'es-ar', b'Argentinian Spanish'), (b'es-co', b'Colombian Spanish'), (b'es-mx', b'Mexican Spanish'), (b'es-ni', b'Nicaraguan Spanish'), (b'es-ve', b'Venezuelan Spanish'), (b'et', b'Estonian'), (b'eu', b'Basque'), (b'fa', b'Persian'), (b'fi', b'Finnish'), (b'fr', b'French'), (b'fy', b'Frisian'), (b'ga', b'Irish'), (b'gd', b'Scottish Gaelic'), (b'gl', b'Galician'), (b'he', b'Hebrew'), (b'hi', b'Hindi'), (b'hr', b'Croatian'), (b'hu', b'Hungarian'), (b'ia', b'Interlingua'), (b'id', b'Indonesian'), (b'io', b'Ido'), (b'is', b'Icelandic'), (b'it', b'Italian'), (b'ja', b'Japanese'), (b'ka', b'Georgian'), (b'kk', b'Kazakh'), (b'km', b'Khmer'), (b'kn', b'Kannada'), (b'ko', b'Korean'), (b'lb', b'Luxembourgish'), (b'lt', b'Lithuanian'), (b'lv', b'Latvian'), (b'mk', b'Macedonian'), (b'ml', b'Malayalam'), (b'mn', b'Mongolian'), (b'mr', b'Marathi'), (b'my', b'Burmese'), (b'nb', b'Norwegian Bokmal'), (b'ne', b'Nepali'), (b'nl', b'Dutch'), (b'nn', b'Norwegian Nynorsk'), (b'os', b'Ossetic'), (b'pa', b'Punjabi'), (b'pl', b'Polish'), (b'pt', b'Portuguese'), (b'pt-br', b'Brazilian Portuguese'), (b'ro', b'Romanian'), (b'ru', b'Russian'), (b'sk', b'Slovak'), (b'sl', b'Slovenian'), (b'sq', b'Albanian'), (b'sr', b'Serbian'), (b'sr-latn', b'Serbian Latin'), (b'sv', b'Swedish'), (b'sw', b'Swahili'), (b'ta', b'Tamil'), (b'te', b'Telugu'), (b'th', b'Thai'), (b'tr', b'Turkish'), (b'tt', b'Tatar'), (b'udm', b'Udmurt'), (b'uk', b'Ukrainian'), (b'ur', b'Urdu'), (b'vi', b'Vietnamese'), (b'zh-hans', b'Simplified Chinese'), (b'zh-hant', b'Traditional Chinese'), (b'tl', 'Tagalog'), (b'rw', 'Kinyarwanda'), (b'ha', 'Hausa')], help_text='Site language', max_length=255, verbose_name='language name'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='global_ga_tracking_code',
            field=models.CharField(blank=True, help_text='Global GA tracking code to be used to view analytics on more than one site globally', max_length=255, null=True, verbose_name='Global GA Tracking Code'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='local_ga_tracking_code',
            field=models.CharField(blank=True, help_text='Local GA tracking code to be used to view analytics on this site only', max_length=255, null=True, verbose_name='Local GA Tracking Code'),
        ),
        migrations.AddField(
            model_name='sectionpage',
            name='content_rotation_end_date',
            field=models.DateTimeField(blank=True, help_text=b'The date rotation will end', null=True),
        ),
        migrations.AddField(
            model_name='sectionpage',
            name='content_rotation_start_date',
            field=models.DateTimeField(blank=True, help_text=b'The date rotation will begin', null=True),
        ),
        migrations.AddField(
            model_name='sectionpage',
            name='friday_rotation',
            field=models.BooleanField(default=False, verbose_name=b'Friday'),
        ),
        migrations.AddField(
            model_name='sectionpage',
            name='monday_rotation',
            field=models.BooleanField(default=False, verbose_name=b'Monday'),
        ),
        migrations.AddField(
            model_name='sectionpage',
            name='saturday_rotation',
            field=models.BooleanField(default=False, verbose_name=b'Saturday'),
        ),
        migrations.AddField(
            model_name='sectionpage',
            name='sunday_rotation',
            field=models.BooleanField(default=False, verbose_name=b'Sunday'),
        ),
        migrations.AddField(
            model_name='sectionpage',
            name='thursday_rotation',
            field=models.BooleanField(default=False, verbose_name=b'Thursday'),
        ),
        migrations.AddField(
            model_name='sectionpage',
            name='time',
            field=wagtail.core.fields.StreamField(((b'time', wagtail.core.blocks.TimeBlock(required=False)),), blank=True, help_text=b'The time/s content will be rotated', null=True),
        ),
        migrations.AddField(
            model_name='sectionpage',
            name='tuesday_rotation',
            field=models.BooleanField(default=False, verbose_name=b'Tuesday'),
        ),
        migrations.AddField(
            model_name='sectionpage',
            name='wednesday_rotation',
            field=models.BooleanField(default=False, verbose_name=b'Wednesday'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='content_rotation_end_date',
            field=models.DateTimeField(blank=True, help_text=b'The date rotation will end', null=True),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='content_rotation_start_date',
            field=models.DateTimeField(blank=True, help_text=b'The date rotation will begin', null=True),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='friday_rotation',
            field=models.BooleanField(default=False, verbose_name=b'Friday'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='monday_rotation',
            field=models.BooleanField(default=False, verbose_name=b'Monday'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='saturday_rotation',
            field=models.BooleanField(default=False, verbose_name=b'Saturday'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='sunday_rotation',
            field=models.BooleanField(default=False, verbose_name=b'Sunday'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='thursday_rotation',
            field=models.BooleanField(default=False, verbose_name=b'Thursday'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='time',
            field=wagtail.core.fields.StreamField(((b'time', wagtail.core.blocks.TimeBlock(required=False)),), blank=True, help_text=b'The time/s content will be rotated', null=True),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='tuesday_rotation',
            field=models.BooleanField(default=False, verbose_name=b'Tuesday'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='wednesday_rotation',
            field=models.BooleanField(default=False, verbose_name=b'Wednesday'),
        ),
        migrations.AlterField(
            model_name='sitelanguage',
            name='locale',
            field=models.CharField(choices=[(b'af', b'Afrikaans'), (b'ar', b'Arabic'), (b'ast', b'Asturian'), (b'az', b'Azerbaijani'), (b'bg', b'Bulgarian'), (b'be', b'Belarusian'), (b'bn', b'Bengali'), (b'br', b'Breton'), (b'bs', b'Bosnian'), (b'ca', b'Catalan'), (b'cs', b'Czech'), (b'cy', b'Welsh'), (b'da', b'Danish'), (b'de', b'German'), (b'el', b'Greek'), (b'en', b'English'), (b'en-au', b'Australian English'), (b'en-gb', b'British English'), (b'eo', b'Esperanto'), (b'es', b'Spanish'), (b'es-ar', b'Argentinian Spanish'), (b'es-co', b'Colombian Spanish'), (b'es-mx', b'Mexican Spanish'), (b'es-ni', b'Nicaraguan Spanish'), (b'es-ve', b'Venezuelan Spanish'), (b'et', b'Estonian'), (b'eu', b'Basque'), (b'fa', b'Persian'), (b'fi', b'Finnish'), (b'fr', b'French'), (b'fy', b'Frisian'), (b'ga', b'Irish'), (b'gd', b'Scottish Gaelic'), (b'gl', b'Galician'), (b'he', b'Hebrew'), (b'hi', b'Hindi'), (b'hr', b'Croatian'), (b'hu', b'Hungarian'), (b'ia', b'Interlingua'), (b'id', b'Indonesian'), (b'io', b'Ido'), (b'is', b'Icelandic'), (b'it', b'Italian'), (b'ja', b'Japanese'), (b'ka', b'Georgian'), (b'kk', b'Kazakh'), (b'km', b'Khmer'), (b'kn', b'Kannada'), (b'ko', b'Korean'), (b'lb', b'Luxembourgish'), (b'lt', b'Lithuanian'), (b'lv', b'Latvian'), (b'mk', b'Macedonian'), (b'ml', b'Malayalam'), (b'mn', b'Mongolian'), (b'mr', b'Marathi'), (b'my', b'Burmese'), (b'nb', b'Norwegian Bokmal'), (b'ne', b'Nepali'), (b'nl', b'Dutch'), (b'nn', b'Norwegian Nynorsk'), (b'os', b'Ossetic'), (b'pa', b'Punjabi'), (b'pl', b'Polish'), (b'pt', b'Portuguese'), (b'pt-br', b'Brazilian Portuguese'), (b'ro', b'Romanian'), (b'ru', b'Russian'), (b'sk', b'Slovak'), (b'sl', b'Slovenian'), (b'sq', b'Albanian'), (b'sr', b'Serbian'), (b'sr-latn', b'Serbian Latin'), (b'sv', b'Swedish'), (b'sw', b'Swahili'), (b'ta', b'Tamil'), (b'te', b'Telugu'), (b'th', b'Thai'), (b'tr', b'Turkish'), (b'tt', b'Tatar'), (b'udm', b'Udmurt'), (b'uk', b'Ukrainian'), (b'ur', b'Urdu'), (b'vi', b'Vietnamese'), (b'zh-hans', b'Simplified Chinese'), (b'zh-hant', b'Traditional Chinese'), (b'zu', 'Zulu'), (b'xh', 'Xhosa'), (b'st', 'Sotho'), (b've', 'Venda'), (b'tn', 'Tswana'), (b'ts', 'Tsonga'), (b'ss', 'Swati'), (b'nr', 'Ndebele')], help_text='Site language', max_length=255, verbose_name='language name'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='show_only_translated_pages',
            field=models.BooleanField(default=False, help_text=b'When selecting this option, untranslated pages will not be visible to the front end user when they viewing a child language of the site'),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='demote_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='feature_as_topic_of_the_day',
            field=models.BooleanField(default=False, help_text='Article to be featured as the Topic of the Day'),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='promote_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='bannerpage',
            name='external_link',
            field=models.TextField(blank=True, help_text=b'External link which a banner will link to. eg https://www.google.co.za/', null=True),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='enable_clickable_tags',
            field=models.BooleanField(default=False, verbose_name=b'Display tags on Front-end'),
        ),
        migrations.RunPython(add_core_permissions_to_groups),
        migrations.AddField(
            model_name='articlepage',
            name='featured_in_homepage_end_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='featured_in_homepage_start_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='featured_in_latest_end_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='featured_in_latest_start_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='featured_in_section_end_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='featured_in_section_start_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='facebook_image',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.Image'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='facebook_sharing',
            field=models.BooleanField(default=False, help_text=b'Enable this field to allow for sharing to Facebook.', verbose_name=b'Facebook'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='social_media_links_on_footer_page',
            field=wagtail.core.fields.StreamField(((b'social_media_site', wagtail.core.blocks.StructBlock(((b'title', wagtail.core.blocks.CharBlock(required=True)), (b'link', wagtail.core.blocks.CharBlock(required=True)), (b'image', wagtail.images.blocks.ImageChooserBlock())))),), blank=True, null=True),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='twitter_image',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.Image'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='twitter_sharing',
            field=models.BooleanField(default=False, help_text=b'Enable this field to allow for sharing to Twitter.', verbose_name=b'Twitter'),
        ),
        migrations.RunPython(set_promote_start_date),
        migrations.RunPython(remove_user_permission_for_moderator_group),
        migrations.RunPython(update_permissions_for_group),
        migrations.CreateModel(
            name='ArticlePageRecommendedSections',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('page', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='recommended_articles', to='core.ArticlePage')),
                ('recommended_article', models.ForeignKey(blank=True, help_text='Recommended articles for this article', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.Page')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='sectionpage',
            name='enable_next_section',
            field=models.BooleanField(default=False, help_text=b"Activate up next section underneath articles in this section will appear with the heading and subheading of that article. The text will say 'next' in order to make the user feel like it's fresh content.", verbose_name=b'Activate up next section underneath articles'),
        ),
        migrations.AddField(
            model_name='sectionpage',
            name='enable_recommended_section',
            field=models.BooleanField(default=False, help_text=b"Underneath the area for 'next articles' recommended articles will appear, with the image + heading + subheading", verbose_name=b'Activate recommended section underneath articles'),
        ),
        migrations.CreateModel(
            name='Languages',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('site', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.Site')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SiteLanguageRelation',
            fields=[
                ('sitelanguage_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.SiteLanguage')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('language_setting', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='languages', to='core.Languages')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
            bases=('core.sitelanguage', models.Model),
        ),
        migrations.RunPython(convert_languages_to_site_language_relation),
        migrations.CreateModel(
            name='ArticlePageTags',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('page', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='nav_tags', to='core.ArticlePage')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Tag',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('feature_in_homepage', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
            bases=(molo.core.models.TranslatablePageMixin, 'wagtailcore.page'),
        ),
        migrations.CreateModel(
            name='TagIndexPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page', molo.core.models.PreventDeleteMixin),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='enable_tag_navigation',
            field=models.BooleanField(default=False, help_text=b'Enable tag navigation. When this is true, the clickable tag functionality will be overriden'),
        ),
        migrations.AddField(
            model_name='articlepagetags',
            name='tag',
            field=models.ForeignKey(blank=True, help_text='Tags for tag navigation', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.Page'),
        ),
        migrations.RunPython(create_tag_index),
        migrations.CreateModel(
            name='SectionPageTags',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('page', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='section_tags', to='core.SectionPage')),
                ('tag', models.ForeignKey(blank=True, help_text='Tags for tag navigation', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.Page')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ArticlePageReactionQuestions',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('page', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='reaction_questions', to='core.ArticlePage')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ReactionQuestion',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
            ],
            options={
                'abstract': False,
            },
            bases=(molo.core.models.TranslatablePageMixin, 'wagtailcore.page'),
        ),
        migrations.CreateModel(
            name='ReactionQuestionChoice',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('image', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.Image')),
            ],
            options={
                'abstract': False,
            },
            bases=(molo.core.models.TranslatablePageMixinNotRoutable, 'wagtailcore.page'),
        ),
        migrations.CreateModel(
            name='ReactionQuestionIndexPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page', molo.core.models.PreventDeleteMixin),
        ),
        migrations.CreateModel(
            name='ReactionQuestionResponse',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('article', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='core.ArticlePage')),
                ('choice', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='core.ReactionQuestionChoice')),
                ('question', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='core.ReactionQuestion')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AddField(
            model_name='articlepagereactionquestions',
            name='reaction_question',
            field=models.ForeignKey(blank=True, help_text='Reaction Questions', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.Page'),
        ),
        migrations.RunPython(create_reaction_question_index),
        migrations.AddField(
            model_name='reactionquestionchoice',
            name='success_image',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.Image'),
        ),
        migrations.AddField(
            model_name='reactionquestionchoice',
            name='success_message',
            field=models.CharField(blank=True, max_length=1000, null=True),
        ),
        migrations.CreateModel(
            name='ArticlePageLanguageProxy',
            fields=[
            ],
            options={
                'verbose_name': 'Article View',
                'proxy': True,
                'verbose_name_plural': 'Article View',
            },
            bases=('core.articlepage',),
        ),
        migrations.AlterField(
            model_name='sitelanguage',
            name='locale',
            field=models.CharField(choices=[(b'af', b'Afrikaans'), (b'ar', b'Arabic'), (b'ast', b'Asturian'), (b'az', b'Azerbaijani'), (b'bg', b'Bulgarian'), (b'be', b'Belarusian'), (b'bn', b'Bengali'), (b'br', b'Breton'), (b'bs', b'Bosnian'), (b'ca', b'Catalan'), (b'cs', b'Czech'), (b'cy', b'Welsh'), (b'da', b'Danish'), (b'de', b'German'), (b'el', b'Greek'), (b'en', b'English'), (b'en-au', b'Australian English'), (b'en-gb', b'British English'), (b'eo', b'Esperanto'), (b'es', b'Spanish'), (b'es-ar', b'Argentinian Spanish'), (b'es-co', b'Colombian Spanish'), (b'es-mx', b'Mexican Spanish'), (b'es-ni', b'Nicaraguan Spanish'), (b'es-ve', b'Venezuelan Spanish'), (b'et', b'Estonian'), (b'eu', b'Basque'), (b'fa', b'Persian'), (b'fi', b'Finnish'), (b'fr', b'French'), (b'fy', b'Frisian'), (b'ga', b'Irish'), (b'gd', b'Scottish Gaelic'), (b'gl', b'Galician'), (b'he', b'Hebrew'), (b'hi', b'Hindi'), (b'hr', b'Croatian'), (b'hu', b'Hungarian'), (b'ia', b'Interlingua'), (b'id', b'Indonesian'), (b'io', b'Ido'), (b'is', b'Icelandic'), (b'it', b'Italian'), (b'ja', b'Japanese'), (b'ka', b'Georgian'), (b'kk', b'Kazakh'), (b'km', b'Khmer'), (b'kn', b'Kannada'), (b'ko', b'Korean'), (b'lb', b'Luxembourgish'), (b'lt', b'Lithuanian'), (b'lv', b'Latvian'), (b'mk', b'Macedonian'), (b'ml', b'Malayalam'), (b'mn', b'Mongolian'), (b'mr', b'Marathi'), (b'my', b'Burmese'), (b'nb', b'Norwegian Bokmal'), (b'ne', b'Nepali'), (b'nl', b'Dutch'), (b'nn', b'Norwegian Nynorsk'), (b'os', b'Ossetic'), (b'pa', b'Punjabi'), (b'pl', b'Polish'), (b'pt', b'Portuguese'), (b'pt-br', b'Brazilian Portuguese'), (b'ro', b'Romanian'), (b'ru', b'Russian'), (b'sk', b'Slovak'), (b'sl', b'Slovenian'), (b'sq', b'Albanian'), (b'sr', b'Serbian'), (b'sr-latn', b'Serbian Latin'), (b'sv', b'Swedish'), (b'sw', b'Swahili'), (b'ta', b'Tamil'), (b'te', b'Telugu'), (b'th', b'Thai'), (b'tr', b'Turkish'), (b'tt', b'Tatar'), (b'udm', b'Udmurt'), (b'uk', b'Ukrainian'), (b'ur', b'Urdu'), (b'vi', b'Vietnamese'), (b'zh-hans', b'Simplified Chinese'), (b'zh-hant', b'Traditional Chinese'), (b'zu', 'Zulu'), (b'xh', 'Xhosa'), (b'st', 'Sotho'), (b've', 'Venda'), (b'tn', 'Tswana'), (b'ts', 'Tsonga'), (b'ss', 'Swati'), (b'nr', 'Ndebele'), (b'bem', 'Bemba'), (b'nya', 'Nyanja'), (b'ny', 'Chichewa')], help_text='Site language', max_length=255, verbose_name='language name'),
        ),
        migrations.AlterField(
            model_name='sitelanguage',
            name='locale',
            field=models.CharField(choices=[(b'af', b'Afrikaans'), (b'ar', b'Arabic'), (b'ast', b'Asturian'), (b'az', b'Azerbaijani'), (b'bg', b'Bulgarian'), (b'be', b'Belarusian'), (b'bn', b'Bengali'), (b'br', b'Breton'), (b'bs', b'Bosnian'), (b'ca', b'Catalan'), (b'cs', b'Czech'), (b'cy', b'Welsh'), (b'da', b'Danish'), (b'de', b'German'), (b'el', b'Greek'), (b'en', b'English'), (b'en-au', b'Australian English'), (b'en-gb', b'British English'), (b'eo', b'Esperanto'), (b'es', b'Spanish'), (b'es-ar', b'Argentinian Spanish'), (b'es-co', b'Colombian Spanish'), (b'es-mx', b'Mexican Spanish'), (b'es-ni', b'Nicaraguan Spanish'), (b'es-ve', b'Venezuelan Spanish'), (b'et', b'Estonian'), (b'eu', b'Basque'), (b'fa', b'Persian'), (b'fi', b'Finnish'), (b'fr', b'French'), (b'fy', b'Frisian'), (b'ga', b'Irish'), (b'gd', b'Scottish Gaelic'), (b'gl', b'Galician'), (b'he', b'Hebrew'), (b'hi', b'Hindi'), (b'hr', b'Croatian'), (b'hu', b'Hungarian'), (b'ia', b'Interlingua'), (b'id', b'Indonesian'), (b'io', b'Ido'), (b'is', b'Icelandic'), (b'it', b'Italian'), (b'ja', b'Japanese'), (b'ka', b'Georgian'), (b'kk', b'Kazakh'), (b'km', b'Khmer'), (b'kn', b'Kannada'), (b'ko', b'Korean'), (b'lb', b'Luxembourgish'), (b'lt', b'Lithuanian'), (b'lv', b'Latvian'), (b'mk', b'Macedonian'), (b'ml', b'Malayalam'), (b'mn', b'Mongolian'), (b'mr', b'Marathi'), (b'my', b'Burmese'), (b'nb', b'Norwegian Bokmal'), (b'ne', b'Nepali'), (b'nl', b'Dutch'), (b'nn', b'Norwegian Nynorsk'), (b'os', b'Ossetic'), (b'pa', b'Punjabi'), (b'pl', b'Polish'), (b'pt', b'Portuguese'), (b'pt-br', b'Brazilian Portuguese'), (b'ro', b'Romanian'), (b'ru', b'Russian'), (b'sk', b'Slovak'), (b'sl', b'Slovenian'), (b'sq', b'Albanian'), (b'sr', b'Serbian'), (b'sr-latn', b'Serbian Latin'), (b'sv', b'Swedish'), (b'sw', b'Swahili'), (b'ta', b'Tamil'), (b'te', b'Telugu'), (b'th', b'Thai'), (b'tr', b'Turkish'), (b'tt', b'Tatar'), (b'udm', b'Udmurt'), (b'uk', b'Ukrainian'), (b'ur', b'Urdu'), (b'vi', b'Vietnamese'), (b'zh-hans', b'Simplified Chinese'), (b'zh-hant', b'Traditional Chinese'), (b'zu', 'Zulu'), (b'xh', 'Xhosa'), (b'st', 'Sotho'), (b've', 'Venda'), (b'tn', 'Tswana'), (b'ts', 'Tsonga'), (b'ss', 'Swati'), (b'nr', 'Ndebele')], help_text='Site language', max_length=255, verbose_name='language name'),
        ),
        migrations.CreateModel(
            name='MoloMedia',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=255, verbose_name='title')),
                ('file', models.FileField(upload_to='media', verbose_name='file')),
                ('type', models.CharField(choices=[('audio', 'Audio file'), ('video', 'Video file')], max_length=255)),
                ('duration', models.PositiveIntegerField(help_text='Duration in seconds', verbose_name='duration')),
                ('width', models.PositiveIntegerField(blank=True, null=True, verbose_name='width')),
                ('height', models.PositiveIntegerField(blank=True, null=True, verbose_name='height')),
                ('thumbnail', models.FileField(blank=True, upload_to='media_thumbnails', verbose_name='thumbnail')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('youtube_link', models.CharField(blank=True, max_length=512, null=True)),
                ('collection', models.ForeignKey(default=wagtail.core.models.get_root_collection_id, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='wagtailcore.Collection', verbose_name='collection')),
                ('tags', taggit.managers.TaggableManager(blank=True, help_text=None, through='taggit.TaggedItem', to='taggit.Tag', verbose_name='tags')),
                ('uploaded_by_user', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='uploaded by user')),
                ('feature_in_homepage', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
                'verbose_name': 'media',
            },
            bases=(wagtail.search.index.Indexed, models.Model),
        ),
        migrations.AlterField(
            model_name='articlepage',
            name='body',
            field=wagtail.core.fields.StreamField(((b'heading', wagtail.core.blocks.CharBlock(classname=b'full title')), (b'paragraph', molo.core.blocks.MarkDownBlock()), (b'image', wagtail.images.blocks.ImageChooserBlock()), (b'list', wagtail.core.blocks.ListBlock(wagtail.core.blocks.CharBlock(label=b'Item'))), (b'numbered_list', wagtail.core.blocks.ListBlock(wagtail.core.blocks.CharBlock(label=b'Item'))), (b'page', wagtail.core.blocks.PageChooserBlock()), (b'media', molo.core.models.MoloMediaBlock(icon=b'media'))), blank=True, null=True),
        ),
        migrations.RunPython(convert_media_to_molo_media),
        migrations.CreateModel(
            name='ImageInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('image_hash', models.CharField(max_length=256, null=True)),
                ('image', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='image_info', to='wagtailimages.Image')),
            ],
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='enable_service_directory',
            field=models.BooleanField(default=False, verbose_name=b'Enable service directory'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='google_places_api_server_key',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='google places server key'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='service_directory_api_base_url',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='service directory base url'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='service_directory_api_password',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='service directory password'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='service_directory_api_username',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='service directory username'),
        ),
        migrations.RunPython(delete_imageinfo),
        migrations.AddField(
            model_name='sitesettings',
            name='fb_analytics_app_id',
            field=models.CharField(blank=True, help_text='The tracking ID to be used to view Facebook Analytics', max_length=25, null=True, verbose_name='Facebook Analytics App ID'),
        ),
        migrations.RunPython(run_wagtail_migration_before_core_34),
        migrations.AddField(
            model_name='bannerpage',
            name='subtitle',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='sitelanguage',
            name='locale',
            field=models.CharField(choices=[('af', 'Afrikaans'), ('ar', 'Arabic'), ('ast', 'Asturian'), ('az', 'Azerbaijani'), ('bg', 'Bulgarian'), ('be', 'Belarusian'), ('bn', 'Bengali'), ('br', 'Breton'), ('bs', 'Bosnian'), ('ca', 'Catalan'), ('cs', 'Czech'), ('cy', 'Welsh'), ('da', 'Danish'), ('de', 'German'), ('dsb', 'Lower Sorbian'), ('el', 'Greek'), ('en', 'English'), ('en-au', 'Australian English'), ('en-gb', 'British English'), ('eo', 'Esperanto'), ('es', 'Spanish'), ('es-ar', 'Argentinian Spanish'), ('es-co', 'Colombian Spanish'), ('es-mx', 'Mexican Spanish'), ('es-ni', 'Nicaraguan Spanish'), ('es-ve', 'Venezuelan Spanish'), ('et', 'Estonian'), ('eu', 'Basque'), ('fa', 'Persian'), ('fi', 'Finnish'), ('fr', 'French'), ('fy', 'Frisian'), ('ga', 'Irish'), ('gd', 'Scottish Gaelic'), ('gl', 'Galician'), ('he', 'Hebrew'), ('hi', 'Hindi'), ('hr', 'Croatian'), ('hsb', 'Upper Sorbian'), ('hu', 'Hungarian'), ('ia', 'Interlingua'), ('id', 'Indonesian'), ('io', 'Ido'), ('is', 'Icelandic'), ('it', 'Italian'), ('ja', 'Japanese'), ('ka', 'Georgian'), ('kk', 'Kazakh'), ('km', 'Khmer'), ('kn', 'Kannada'), ('ko', 'Korean'), ('lb', 'Luxembourgish'), ('lt', 'Lithuanian'), ('lv', 'Latvian'), ('mk', 'Macedonian'), ('ml', 'Malayalam'), ('mn', 'Mongolian'), ('mr', 'Marathi'), ('my', 'Burmese'), ('nb', 'Norwegian Bokmål'), ('ne', 'Nepali'), ('nl', 'Dutch'), ('nn', 'Norwegian Nynorsk'), ('os', 'Ossetic'), ('pa', 'Punjabi'), ('pl', 'Polish'), ('pt', 'Portuguese'), ('pt-br', 'Brazilian Portuguese'), ('ro', 'Romanian'), ('ru', 'Russian'), ('sk', 'Slovak'), ('sl', 'Slovenian'), ('sq', 'Albanian'), ('sr', 'Serbian'), ('sr-latn', 'Serbian Latin'), ('sv', 'Swedish'), ('sw', 'Swahili'), ('ta', 'Tamil'), ('te', 'Telugu'), ('th', 'Thai'), ('tr', 'Turkish'), ('tt', 'Tatar'), ('udm', 'Udmurt'), ('uk', 'Ukrainian'), ('ur', 'Urdu'), ('vi', 'Vietnamese'), ('zh-hans', 'Simplified Chinese'), ('zh-hant', 'Traditional Chinese'), (b'zu', 'Zulu'), (b'xh', 'Xhosa'), (b'st', 'Sotho'), (b've', 'Venda'), (b'tn', 'Tswana'), (b'ts', 'Tsonga'), (b'ss', 'Swati'), (b'nr', 'Ndebele')], help_text='Site language', max_length=255, verbose_name='language name'),
        ),
        migrations.CreateModel(
            name='Timezone',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=255, unique=True)),
            ],
        ),
        migrations.RunPython(seed_timezones, delete_timezones),
        migrations.CreateModel(
            name='CmsSettings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('site', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.Site')),
                ('timezone', models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to='core.Timezone')),
            ],
            options={
                'verbose_name': 'CMS settings',
            },
        ),
        migrations.RunPython(set_initial_timezone, unset_timezone),
        migrations.CreateModel(
            name='MoloPage',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
            },
            bases=('wagtailcore.page',),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-09-06 09:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0004_add_core_translation_fields'),
    ]

    operations = [
        migrations.AddField(
            model_name='sitesettings',
            name='enable_multi_category_service_directory_search',
            field=models.BooleanField(default=False, verbose_name=b'Enable multi service directory search'),
        )
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-02-22 07:27
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0011_hero_of_the_day_name_change'),
    ]

    operations = [
        migrations.AlterField(
            model_name='articlepage',
            name='feature_as_hero_article',
            field=models.BooleanField(default=False, help_text='Article to be featured as the Hero Article'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-09-14 12:51
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0043_sitesettings_show_only_translated_pages'),
    ]

    operations = [
        migrations.AddField(
            model_name='articlepage',
            name='demote_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='feature_as_topic_of_the_day',
            field=models.BooleanField(default=False, help_text='Article to be featured as the Topic of the Day'),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='promote_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



from os.path import join

from django.test import TestCase, override_settings
from django.conf import settings

from wagtail.images.tests.utils import Image, get_test_image_file

from molo.core.models import ImageInfo
from molo.core.tests.base import MoloTestCaseMixin
from molo.core.tests.constants import TEST_IMAGE_HASH


@override_settings(MEDIA_ROOT=join(settings.PROJECT_ROOT, 'media'))
@override_settings(
    DEFAULT_FILE_STORAGE='django.core.files.storage.FileSystemStorage')
class TestImageInfoObject(MoloTestCaseMixin, TestCase):
    def setUp(self):
        self.mk_main()

    def test_saving_image_creates_image_info(self):
        self.assertEquals(Image.objects.count(), 0)
        self.assertEquals(ImageInfo.objects.count(), 0)
        # creates the image which includes saving the image
        image = Image.objects.create(
            title="Test image",
            file=get_test_image_file(),
        )

        # post save of image should create the image info
        self.assertEquals(Image.objects.count(), 1)
        self.assertEquals(ImageInfo.objects.count(), 1)

        # check image hash in image info is correct
        image_info = ImageInfo.objects.first()
        self.assertEquals(image_info.image_hash, TEST_IMAGE_HASH)
        self.assertEquals(image_info.image, image)

    def test_attaching_image_info(self):
        # creates the image which includes saving the image
        image = Image.objects.create(
            title="Test image",
            file=get_test_image_file(),
        )
        self.assertEquals(ImageInfo.objects.count(), 1)
        image.image_info.delete()

        # deleting image info should not delete the image
        self.assertEquals(ImageInfo.objects.count(), 0)
        self.assertEquals(Image.objects.count(), 1)

        # add image info to image with no image info
        image_info = ImageInfo.objects.create(image=image)

        self.assertEquals(ImageInfo.objects.count(), 1)

        image_info.refresh_from_db()
        # check image hash in image info is correct
        self.assertEquals(image_info.image_hash, TEST_IMAGE_HASH)
        self.assertEquals(image_info.image, image)

    def test_attaching_image_info_save_image_twice(self):
        # creates the image which includes saving the image
        image = Image.objects.create(
            title="Test image",
            file=get_test_image_file(),
        )
        self.assertEquals(ImageInfo.objects.count(), 1)
        image.save()
        # confirm that we're not creating an ImageInfo
        # object each time
        self.assertEquals(ImageInfo.objects.count(), 1)

    def test_image_info_deleted_with_image(self):
        image = Image.objects.create(
            title="Test image",
            file=get_test_image_file(),
        )
        self.assertEquals(Image.objects.count(), 1)
        self.assertEquals(ImageInfo.objects.count(), 1)

        image.delete()

        self.assertEquals(Image.objects.count(), 0)
        self.assertEquals(ImageInfo.objects.count(), 0)

    def test_image_info_updated_on_file_change(self):
        file_1 = get_test_image_file(colour='black', size=(100, 100))
        file_2 = get_test_image_file(colour='white', size=(80, 80))

        image = Image.objects.create(
            title="Test image",
            file=file_1,
        )

        id_ = image.id
        original_hash = image.image_info.image_hash

        image.file = file_2
        image.save()

        updated_image = Image.objects.get(id=id_)

        new_hash = updated_image.image_info.image_hash

        self.assertNotEqual(
            original_hash,
            new_hash
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-08-17 19:06
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0042_content_rotation_dates_and_times'),
    ]

    operations = [
        migrations.AddField(
            model_name='sitesettings',
            name='show_only_translated_pages',
            field=models.BooleanField(default=False, help_text=b'When selecting this option, untranslated pages will not be visible to the front end user when they viewing a child language of the site'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


def create_form_index(apps, schema_editor):
    from molo.core.models import FormIndexPage, Main
    main = Main.objects.first()

    if main:
        form_index = FormIndexPage(title='Forms', slug='forms')
        main.add_child(instance=form_index)
        form_index.save_revision().publish()


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0007_add_contact_form'),
    ]

    operations = [
        migrations.RunPython(create_form_index),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-08-31 13:12
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0002_add_can_view_response_reaction_question_permission'),
    ]

    operations = [
        migrations.AddField(
            model_name='sitesettings',
            name='default_service_directory_radius',
            field=models.PositiveSmallIntegerField(blank=True, help_text='When set this will enable service directory radius filter as the set value for the default radius  value', null=True, verbose_name='Default Service Directory Radius'),
        )
    ]



from itertools import chain
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.core.cache import cache
from django import template
from django.utils.safestring import mark_safe
from django.db.models import Case, When
from markdown import markdown

from molo.core.models import (
    Page, ArticlePage, SectionPage, SiteSettings, Languages, Tag,
    ArticlePageTags, SectionIndexPage, ReactionQuestion,
    ReactionQuestionChoice, BannerPage, get_translation_for)

from prometheus_client import Summary

register = template.Library()


REQUEST_TIME = Summary(
        'request_processing_seconds', 'Time spent processing request')


def get_language(site, locale):
    language_cache_key = 'get_pages_language_{}_{}'.format(site.pk, locale)
    language = cache.get(language_cache_key)
    if not language:
        language = Languages.for_site(site).languages.filter(
            locale=locale).first()
        cache.set(language_cache_key, language, None)
    return language


def get_pages(context, queryset, locale):
    from molo.core.models import get_translation_for

    if queryset.count() == 0:
        return []

    request = context['request']
    if not hasattr(request, 'site'):
        return list[queryset]

    language = get_language(request.site, locale)
    if language and language.is_main_language:
        return list(queryset.live())
    pages = get_translation_for(queryset, locale, request.site)
    return pages or []


@register.assignment_tag(takes_context=True)
def load_tags(context):
    request = context['request']
    locale = context.get('locale_code')

    if request.site:
        qs = Tag.objects.descendant_of(request.site.root_page).filter(
            language__is_main_language=True).live()
    else:
        return[]

    return get_pages(context, qs, locale)


@register.assignment_tag(takes_context=True)
def load_sections(context):
    request = context['request']
    locale = context.get('locale_code')
    if request.site:
        qs = request.site.root_page.specific.sections()
    else:
        return []
    return get_pages(context, qs, locale)


@register.assignment_tag(takes_context=True)
def get_translation(context, page):
    locale_code = context.get('locale_code')
    try:
        translation = page.translated_pages.get(language__locale=locale_code)
    except:
        return page

    return translation


@register.assignment_tag(takes_context=True)
def get_parent(context, page):
    parent = page.get_parent()
    if not parent.specific_class == SectionIndexPage:
        return get_translation(context, parent.specific)
    return None


@register.inclusion_tag(
    'core/tags/section_listing_homepage.html',
    takes_context=True
)
def section_listing_homepage(context):
    locale_code = context.get('locale_code')
    return {
        'sections': load_sections(context),
        'request': context['request'],
        'locale_code': locale_code,
    }


@register.inclusion_tag(
    'core/tags/tag_menu_homepage.html',
    takes_context=True
)
def tag_menu_homepage(context):
    locale_code = context.get('locale_code')

    return {
        'tags': load_tags(context),
        'request': context['request'],
        'locale_code': locale_code,
    }


@register.inclusion_tag(
    'core/tags/latest_listing_homepage.html',
    takes_context=True
)
def latest_listing_homepage(context, num_count=5):
    request = context['request']
    locale = context.get('locale_code')

    if request.site:
        articles = request.site.root_page.specific.latest_articles()
    else:
        articles = []
    return {
        'articles': get_pages(context, articles, locale)[:num_count],
        'request': context['request'],
        'locale_code': locale,
    }


@register.inclusion_tag(
    'core/tags/hero_article.html',
    takes_context=True
)
def hero_article(context):
    request = context['request']
    locale = context.get('locale_code')

    if request.site:
        articles = request.site.root_page.specific \
            .hero_article()
    else:
        articles = ArticlePage.objects.none()

    return {
        'articles': get_pages(context, articles, locale),
        'request': context['request'],
        'locale_code': locale,
    }


@register.inclusion_tag('core/tags/bannerpages.html', takes_context=True)
def bannerpages(context, position=-1):
    request = context['request']
    locale = context.get('locale_code')

    if request.site:
        pages = request.site.root_page.specific.bannerpages().exact_type(
            BannerPage)
    else:
        pages = []
    if position >= 0:
        banners = get_pages(context, pages, locale)
        if position > (len(banners) - 1):
            return None
        banners = get_pages(context, pages, locale)
        if banners and len(banners) > position:
            return {
                'bannerpages': [banners[position]],
                'request': context['request'],
                'locale_code': locale,
            }
        return None
    return {
        'bannerpages': get_pages(context, pages, locale),
        'request': context['request'],
        'locale_code': locale,
    }


@register.inclusion_tag('core/tags/footerpage.html', takes_context=True)
def footer_page(context):
    request = context['request']
    locale = context.get('locale_code')

    if request.site:
        pages = request.site.root_page.specific.footers()
    else:
        pages = []

    return {
        'footers': get_pages(context, pages, locale),
        'request': context['request'],
        'locale_code': locale,
    }


@register.inclusion_tag('core/tags/breadcrumbs.html', takes_context=True)
def breadcrumbs(context):
    self = context.get('self')
    locale_code = context.get('locale_code')
    ancestors = []
    translated_ancestors = []
    if self is not None and not self.depth <= 2:
        ancestors = Page.objects.live().ancestor_of(
            self, inclusive=True).filter(depth__gt=3)
        if ancestors:
            translated_ancestors = get_pages(context, ancestors, locale_code)

    return {
        'ancestors': translated_ancestors,
        'request': context['request'],
    }


@register.inclusion_tag(
    'wagtail/translations_actions.html', takes_context=True)
def render_translations(context, page):
    from molo.core.models import TranslatablePageMixinNotRoutable
    if not issubclass(type(page.specific), TranslatablePageMixinNotRoutable):
        return {}
    if not page.specific.language.is_main_language:
        return {}

    languages = [
        (l.locale, str(l))
        for l in Languages.for_site(context['request'].site).languages.filter(
            is_main_language=False)]

    translated = []
    for code, title in languages:
        if page.specific.translated_pages.filter(
                language__locale=code).exists():
            translated.append({
                'locale': {'title': title, 'code': code},
                'translated':
                    page.specific.translated_pages.filter(
                        language__locale=code).first()})
        else:
            translated.append({
                'locale': {'title': title, 'code': code},
                'translated': []})
    return {
        'translations': translated,
        'page': page
    }


@register.assignment_tag(takes_context=True)
def load_descendant_articles_for_section(
        context, section, featured_in_homepage=None, featured_in_section=None,
        featured_in_latest=None, count=5):
    '''
    Returns all descendant articles (filtered using the parameters)
    If the `locale_code` in the context is not the main language, it will
    return the translations of the live articles.
    '''
    page = section.get_main_language_page()
    locale = context.get('locale_code')

    qs = ArticlePage.objects.descendant_of(page).filter(
        language__is_main_language=True)

    if featured_in_homepage is not None:
        qs = qs.filter(featured_in_homepage=featured_in_homepage).order_by(
            '-featured_in_homepage_start_date')

    if featured_in_latest is not None:
        qs = qs.filter(featured_in_latest=featured_in_latest)

    if featured_in_section is not None:
        qs = qs.filter(featured_in_section=featured_in_section).order_by(
            '-featured_in_section_start_date')

    if not locale:
        return qs.live()[:count]

    return get_pages(context, qs, locale)[:count]


@register.assignment_tag(takes_context=True)
def load_child_articles_for_section(context, section, count=5):
    '''
    Returns all child articles
    If the `locale_code` in the context is not the main language, it will
    return the translations of the live articles.
    '''
    locale = context.get('locale_code')
    main_language_page = section.get_main_language_page()

    # TODO: Consider caching the pks of these articles using a timestamp on
    # section as the key so tha twe don't always do these joins

    child_articles = ArticlePage.objects.child_of(
        main_language_page).filter(
        language__is_main_language=True).order_by(
        '-first_published_at')

    related_articles = ArticlePage.objects.filter(
        related_sections__section__slug=main_language_page.slug)
    qs = list(chain(
        get_pages(context, child_articles, locale),
        get_pages(context, related_articles, locale)))

    # Pagination
    if count:
        p = context.get('p', 1)
        paginator = Paginator(qs, count)

        try:
            articles = paginator.page(p)
        except PageNotAnInteger:
            articles = paginator.page(1)
        except EmptyPage:
            articles = paginator.page(paginator.num_pages)
    else:
        articles = qs
    if not locale:
        return articles

    context.update({'articles_paginated': articles})
    return articles


def get_articles_for_tags_with_translations(
        request, tag, exclude_list, locale, context, exclude_pks):

    pks = [article_tag.page.pk for article_tag in
           ArticlePageTags.objects.filter(tag=tag)]
    return get_pages(
        context, ArticlePage.objects.descendant_of(
            request.site.root_page).filter(pk__in=pks).exclude(
                pk__in=exclude_pks), locale)


@register.assignment_tag(takes_context=True)
def get_articles_for_tag(context, tag):
    request = context['request']
    locale = context.get('locale_code')
    if tag:
        main_tag = tag.specific.get_main_language_page()
        pks = [article_tag.page.pk for article_tag in
               ArticlePageTags.objects.filter(tag=main_tag)]
        return get_pages(
            context, ArticlePage.objects.descendant_of(
                request.site.root_page).filter(
                pk__in=pks).order_by(
                '-first_published_at'), locale)
    return None


@register.assignment_tag(takes_context=True)
def get_next_tag(context, tag):
    request = context['request']
    locale_code = context.get('locale_code')
    current_tag = tag.get_main_language_page()
    qs = Tag.objects.descendant_of(request.site.root_page).filter(
        language__is_main_language=True).live()
    if qs.exists():
        tags = list(qs)
        if not (len(tags) == tags.index(current_tag) + 1):
            next_tag = tags[tags.index(current_tag) + 1]
        else:
            next_tag = tags[0]

        next_tag_translated = get_translation_for(
                [next_tag], locale_code, context['request'].site)
        if next_tag_translated:
            return next_tag_translated[0]
        else:
            return next_tag


@register.assignment_tag(takes_context=True)
def get_tags_for_section(context, section, tag_count=2, tag_article_count=4):
    request = context['request']
    locale = context.get('locale_code')
    # Featured Tags
    exclude_pks = []
    tags_list = []
    main_language_page = section.get_main_language_page()
    child_articles = ArticlePage.objects.descendant_of(
        main_language_page).filter(language__is_main_language=True)
    for article in child_articles:
        exclude_pks.append(article.pk)
    related_articles = ArticlePage.objects.filter(
        related_sections__section__slug=main_language_page.slug)
    for article in related_articles:
        exclude_pks.append(article.pk)

    tags = [
        section_tag.tag.pk for section_tag in
        section.get_main_language_page().specific.section_tags.all()
        if section_tag.tag]
    if tags and request.site:
        qs = Tag.objects.descendant_of(
            request.site.root_page).live().filter(pk__in=tags)
        for tag in qs:
            tag_articles = get_articles_for_tags_with_translations(
                request, tag, exclude_pks, locale, context,
                exclude_pks)
            if len(tag_articles) > tag_article_count:
                tag_articles = tag_articles[:tag_article_count]
            exclude_pks += [p.pk for p in tag_articles]
            tags = get_pages(context, qs.filter(pk=tag.pk), locale)
            if tags:
                tags_list.append((
                    tags[0],
                    tag_articles))
    else:
        return []

    return tags_list


@REQUEST_TIME.time()
@register.simple_tag(takes_context=True)
def get_tag_articles(
        context, section_count=1, tag_count=4, sec_articles_count=4,
        latest_article_count=3):
    # TODO: consider caching this tag - these queries are too expensive

    request = context['request']
    locale = context.get('locale_code')

    exclude_pks = []
    data = {}
    tags_list = []
    sections_list = []
    latest_articles = []
    # get x amount of articles featured in latest
    # ordered by featured in latest promote date
    all_latest_articles = ArticlePage.objects.descendant_of(
        request.site.root_page).filter(
        language__is_main_language=True,
        featured_in_latest=True).exact_type(ArticlePage).exclude(
            pk__in=exclude_pks).order_by('-featured_in_latest_start_date')
    if all_latest_articles:
        all_translated_latest_articles = get_pages(
            context, all_latest_articles, locale)
        if all_translated_latest_articles:
            if len(all_translated_latest_articles) >= latest_article_count:
                latest_articles = all_translated_latest_articles[
                    :latest_article_count]
                exclude_pks += [p.pk for p in latest_articles]
            else:
                latest_articles = all_translated_latest_articles
                exclude_pks += [p.pk for p in latest_articles]
        elif len(all_latest_articles) >= latest_article_count:
            latest_articles = all_latest_articles[:latest_article_count]
            exclude_pks += [p.pk for p in latest_articles]
        else:
            latest_articles = list(all_latest_articles)
            exclude_pks += [p.pk for p in latest_articles]

    # Featured Section/s
    sections = request.site.root_page.specific.sections()
    for section in sections[:section_count]:
        sec_articles = ArticlePage.objects.descendant_of(section).filter(
            language__is_main_language=True,
            featured_in_homepage=True).order_by(
                '-featured_in_homepage_start_date').exclude(
                pk__in=exclude_pks)
        exclude_pks += [p.pk for p in sec_articles[:sec_articles_count]]
        section = SectionPage.objects.filter(pk=section.pk)
        section_for_locale = get_pages(context, section, locale)
        if section_for_locale:
            section = section_for_locale[0]
        sec_articles_for_locale = get_pages(context, sec_articles, locale)
        if sec_articles_for_locale and len(
                sec_articles_for_locale) > sec_articles_count:
            sec_articles_for_locale = sec_articles_for_locale[
                :sec_articles_count]
        if section_for_locale:
            sections_list.append((
                section,
                sec_articles_for_locale))
    data.update({'sections': sections_list})

    # Featured Tag/s
    tag_qs = Tag.objects.descendant_of(request.site.root_page).filter(
        feature_in_homepage=True).live()
    for tag in tag_qs:
        tag_articles = get_articles_for_tags_with_translations(
            request, tag, exclude_pks, locale,
            context, exclude_pks)
        if tag_articles and len(tag_articles) > tag_count:
            tag_articles = tag_articles[:tag_count]
        exclude_pks += [p.pk for p in tag_articles]
        tag_for_locale = get_pages(context, tag_qs.filter(pk=tag.pk), locale)
        if tag_for_locale:
            tags_list.append((
                tag_for_locale[0],
                tag_articles))
        else:
            tags_list.append((
                tag,
                tag_articles))
    data.update({'tags_list': tags_list})

    # Latest Articles
    data.update({
        'latest_articles': latest_articles + get_pages(
            context, ArticlePage.objects.descendant_of(
                request.site.root_page).filter(
                language__is_main_language=True).exact_type(
                    ArticlePage).exclude(pk__in=exclude_pks).order_by(
                        '-featured_in_latest'), locale)})

    return data


@register.assignment_tag(takes_context=True)
def load_tags_for_article(context, article):
    if not article.specific.__class__ == ArticlePage:
        return None

    locale = context.get('locale_code')
    request = context['request']

    cache_key = "load_tags_for_article_{}_{}_{}_{}".format(
        locale, request.site.pk, article.pk,
        article.latest_revision_created_at.isoformat())
    tags_pks = cache.get(cache_key)

    if not tags_pks:
        tags = [
            article_tag.tag.pk for article_tag in
            article.specific.get_main_language_page().nav_tags.all()
            if article_tag.tag]
        if tags and request.site:
            qs = Tag.objects.descendant_of(
                request.site.root_page).live().filter(pk__in=tags)
            tags_pks = qs.values_list("pk", flat=True)
            cache.set(cache_key, tags_pks, 300)
        else:
            return []

    qs = Tag.objects.descendant_of(
        request.site.root_page).live().filter(pk__in=tags_pks)
    return get_pages(context, qs, locale)


@register.assignment_tag(takes_context=True)
def load_choices_for_reaction_question(context, question):
    locale = context.get('locale_code')
    if question:
        question = question.specific.get_main_language_page().specific
    if question and question.get_children():
        choices = ReactionQuestionChoice.objects.child_of(
            question).filter(language__is_main_language=True)
        return get_pages(context, choices, locale)
    return []


@register.assignment_tag(takes_context=True)
def load_user_can_vote_on_reaction_question(context, question, article_pk):
    request = context['request']
    if question:
        question = question.specific.get_main_language_page()
        article = ArticlePage.objects.get(pk=article_pk)
        if hasattr(article, 'get_main_language_page'):
            article = article.get_main_language_page()
        return not question.has_user_submitted_reaction_response(
            request, question.pk, article.pk)


@register.assignment_tag(takes_context=True)
def load_reaction_question(context, article):
    locale = context.get('locale_code')
    request = context['request']
    question = None
    if article:
        article_question = article.get_main_language_page() \
            .reaction_questions.all().first()
        if hasattr(article_question, 'reaction_question'):
            question = article_question.reaction_question

        if question and request.site:
            qs = ReactionQuestion.objects.descendant_of(
                request.site.root_page).live().filter(
                    pk=question.pk, language__is_main_language=True)
        else:
            return []
        translated_question = get_pages(context, qs, locale)
        if translated_question:
            return get_pages(context, qs, locale)[0]
        return question


@register.assignment_tag(takes_context=True)
def load_child_sections_for_section(context, section, count=None):
    '''
    Returns all child sections
    If the `locale_code` in the context is not the main language, it will
    return the translations of the live articles.
    '''
    page = section.get_main_language_page()
    locale = context.get('locale_code')

    qs = SectionPage.objects.child_of(page).filter(
        language__is_main_language=True)

    if not locale:
        return qs[:count]
    return get_pages(context, qs, locale)


@register.assignment_tag(takes_context=True)
def load_sibling_sections(context, section, count=None):
    '''
    Returns all sibling sections
    If the `locale_code` in the context is not the main language, it will
    return the translations of the live articles.
    '''
    page = section.get_main_language_page()
    locale = context.get('locale_code')

    qs = SectionPage.objects.sibling_of(page).filter(
        language__is_main_language=True)

    if not locale:
        return qs[:count]
    return get_pages(context, qs, locale)


@register.filter
def handle_markdown(value):
    md = markdown(
        value,
        extensions=[
            'markdown.extensions.fenced_code',
            'codehilite',
        ]
    )
    """ For some unknown reason markdown wraps the value in <p> tags.
        Currently there doesn't seem to be an extension to turn this off.
    """
    open_tag = '<p>'
    close_tag = '</p>'
    if md.startswith(open_tag) and md.endswith(close_tag):
        md = md[len(open_tag):-len(close_tag)]
    return mark_safe(md)


@register.inclusion_tag(
    'core/tags/social_media_footer.html',
    takes_context=True
)
def social_media_footer(context, page=None):
    request = context['request']
    locale = context.get('locale_code')
    social_media = SiteSettings.for_site(request.site).\
        social_media_links_on_footer_page

    data = {
        'social_media': social_media,
        'request': context['request'],
        'locale_code': locale,
        'page': page,
    }
    return data


@register.inclusion_tag(
    'core/tags/social_media_article.html',
    takes_context=True
)
def social_media_article(context, page=None):
    request = context['request']
    locale = context.get('locale_code')
    site_settings = SiteSettings.for_site(request.site)

    if site_settings.facebook_sharing:
        facebook = site_settings.facebook_image
    else:
        facebook = False

    if site_settings.twitter_sharing:
        twitter = site_settings.twitter_image
    else:
        twitter = False

    if site_settings.whatsapp_sharing:
        whatsapp = site_settings.whatsapp_image
    else:
        whatsapp = False

    if site_settings.viber_sharing:
        viber = site_settings.viber_image
    else:
        viber = False

    if site_settings.telegram_sharing:
        telegram = site_settings.telegram_image
    else:
        telegram = False

    data = {
        'facebook': facebook,
        'twitter': twitter,
        'whatsapp': whatsapp,
        'viber': viber,
        'telegram': telegram,
        'request': context['request'],
        'locale_code': locale,
        'page': page,
    }
    return data


@register.assignment_tag(takes_context=True)
def get_next_article(context, article):
    locale_code = context.get('locale_code')
    section = article.get_parent_section()
    articles = load_child_articles_for_section(context, section, count=None)
    if len(articles) > 1:
        if len(articles) > articles.index(article) + 1:
            next_article = articles[articles.index(article) + 1]
        else:
            next_article = articles[0]
    else:
        return None
    try:
        return next_article.translated_pages.get(language__locale=locale_code)
    except:
        if next_article.language.locale == locale_code or not \
                SiteSettings.for_site(
                    context['request'].site).show_only_translated_pages:
            return next_article
        return None


@register.assignment_tag(takes_context=True)
def get_recommended_articles(context, article):
    locale_code = context.get('locale_code')

    if article.recommended_articles.all():
        recommended_articles = article.recommended_articles.all()
    else:
        a = article.get_main_language_page()
        recommended_articles = a.specific.recommended_articles.all()
    # http://stackoverflow.com/questions/4916851/django-get-a-queryset-from-array-of-ids-in-specific-order/37648265#37648265 # noqa
    # the following allows us to order the results of the querystring
    recommended_articles_queryset = recommended_articles.values_list(
        'recommended_article', flat=True)

    if not recommended_articles_queryset:
        return []
    preserved = Case(
        *[When(pk=pk, then=pos) for pos, pk in enumerate(
                recommended_articles_queryset)])
    articles = ArticlePage.objects.filter(
                  pk__in=recommended_articles_queryset).order_by(preserved)

    return get_pages(context, articles, locale_code)


@register.simple_tag(takes_context=True)
def should_hide_delete_button(context, page):
    return hasattr(page.specific, 'hide_delete_button')
